@@ -87,17 +87,24 @@ enum ej_proptype {
 	EJ_PROP_NULL,
 };
 
-/* ctype macros, when you're passing a byte,
- * and you don't want to worry about whether it's char or uchar.
- * Call the regular routines when c is an int, from fgetc etc. */
-#define isspaceByte(c) isspace((uchar)c)
-#define isalphaByte(c) isalpha((uchar)c)
-#define isalnumByte(c) isalnum((uchar)c)
-#define islowerByte(c) islower((uchar)c)
-#define isupperByte(c) isupper((uchar)c)
-#define isdigitByte(c) isdigit((uchar)c)
-#define ispunctByte(c) ispunct((uchar)c)
-#define isprintByte(c) isprint((uchar)c)
+/*********************************************************************
+ctype macros, when you're passing a byte, and you don't want to worry
+about whether it's char or uchar. Remember there is no standard,
+char could be signed or unsigned depending on the compiler.
+And what the hell is the 0x80 test in isspaceByte for?
+On some platforms, under setlocale(LC_ALL, ""),
+isspace(0xa0) returns true. That is, the breakspace.
+We never want this to be true in the context of edbrowse.
+*********************************************************************/
+
+#define isspaceByte(c) (isspace((uchar)(c)) && !((c)&0x80))
+#define isalphaByte(c) isalpha((uchar)(c))
+#define isalnumByte(c) isalnum((uchar)(c))
+#define islowerByte(c) islower((uchar)(c))
+#define isupperByte(c) isupper((uchar)(c))
+#define isdigitByte(c) isdigit((uchar)(c))
+#define ispunctByte(c) ispunct((uchar)(c))
+#define isprintByte(c) isprint((uchar)(c))
 
 /* http encoding, content type, content transfer encoding */
 enum { ENC_PLAIN, ENC_COMPRESS, ENC_GZIP, ENC_URL, ENC_MFD };
@@ -187,6 +194,7 @@ struct i_get {
 	const char *thisfile;
 	char *referrer;
 	CURL *h;
+	struct ebFrame *cf; // current frame
 	int tsn; // thread sequence number
 // State of download to disk, see http.c for state values.
 	int down_state;
@@ -262,14 +270,14 @@ struct DBTABLE {
 extern CURL *global_http_handle;
 extern CURLSH *global_share_handle;
 extern int debugLevel;		/* 0 to 9 */
-extern bool debugClone, debugEvent, debugThrow, debugCSS, dhs, debugLayout;
+extern char *debugFileName;
+extern FILE *debugFile;
+extern bool debugClone, debugEvent, debugThrow, debugCSS, debugScanner, debugLayout;
 extern bool demin; // deminimize javascript
 extern bool uvw; // trace points
 extern bool gotimers; // run javascript timers
 extern int timerspeed; // slowdown factor for javascript timers
 extern int rr_interval; // rerender the screen after this many seconds
-extern FILE *debugFile;
-extern char *debugFileName;
 extern char *sslCerts;		// ssl certificates to validate the secure server
 extern char *pubKey;		// publick key file for ssh etc
 extern int verifyCertificates;	/* is a certificate required for the ssl connection? */
@@ -323,6 +331,7 @@ extern bool fetchBlobColumns, ls_reverse;
 extern uchar ls_sort, dirWrite;
 extern char lsformat[12];
 extern bool dno; // directory names only
+extern bool ebvar; // update environment variables before each shell command
 extern bool flow; // send mail with format=flowed
 extern bool caseInsensitive, searchStringsAll, searchWrap, ebre;
 extern bool allowRedirection;	/* from http code 301, or http refresh */
@@ -336,6 +345,7 @@ extern bool showHiddenFiles;	/* during directory scan */
 extern bool showall; // show messages that are normally hidden, for various reasons
 extern bool doColors;
 extern int context;		// which session (buffer) are we in?
+extern int cx_previous;		// the number of the previous session
 extern pst linePending;
 extern char *changeFileName;
 extern char *addressFile;	// your address book
@@ -395,6 +405,7 @@ struct ebFrame {
 	bool uriEncoded; // filename is url encoded
 	bool jslink; // linke to javascript
 	bool browseMode;
+	bool xmlMode;
 	short jtmin;
 	char *dw;		// document.write string
 	int dw_l;		// length of the above
@@ -431,7 +442,7 @@ struct ebWindow {
 	int dot, dol;
 // remember dot and dol for the raw text, when in browse mode
 	int r_dot, r_dol;
-	int f_dot; // foreward dot, in case we browse again
+	int f_dot; // forward dot, in case we browse again
 	struct ebFrame f0; /* first frame */
 	struct ebFrame *jdb_frame; // if in jdb mode
 	char *referrer; // another web page that brought this one to life
@@ -485,9 +496,12 @@ int ehsn;
 	bool sqlMode:1;		// accessing a table
 	bool irciMode:1;		// input irc
 	bool ircoMode:1;		// output irc
-	bool ircChannels; // show channel when displaying messages
-	FILE *ircF;
-	ushort ircOther;
+	bool ircChannels:1; // show channel when displaying messages
+	bool ircSecure:1; // ssl connection
+	bool ircPingOut:1; // sent ping to server
+	int irc_fd; // file descriptor for socket
+	void *irc_ssl; // stream for a secure socket
+	ushort ircOther; // point to the output side
 	ushort ircCount;
 	char *ircNick, *ircChannel;
 	time_t ircRespond;
@@ -517,14 +531,15 @@ If ints are larger then I don't even use this constant.
 
 /* An edit session */
 struct ebSession {
-	Window *fw, *lw;	/* first window, last window */
+	Window *fw, *lw;	// first window, last window
+	Window *fw2, *lw2;	// second chunk of the stack
 };
 extern struct ebSession sessionList[];
 extern struct ebSession *cs;	/* current session */
 extern int maxSession;
 
 // The information on an html tag
-#define MAXTAGNAME 20
+#define MAXTAGNAME 24
 struct tagInfo {
 	const char name[MAXTAGNAME];
 	const char *desc;
@@ -569,7 +584,10 @@ struct htmlTag {
 	long hcode;
 	bool loadsuccess;
 	uchar step; // prerender, decorate, load script, runscript
-	bool slash:1;		/* as in </A> */
+// the slash member was for the tag coresponding to <foo/>, the closing tag,
+// but we don't do that any more, just one tag for <foo> in the tree.
+// Except ... <pre> and </pre> need separate tags.
+	bool slash:1;
 	bool textin:1; /* <a> some text </a> */
 	bool deleted:1; /* deleted from the current buffer */
 	bool dead:1; // removed by garbage collection
@@ -606,6 +624,8 @@ struct htmlTag {
 	bool iscolor:1;
 	bool ur:1;		// row unfolded, only for trf
 	bool inur:1;		// in ur command
+	bool threadcreated:1;
+	bool threadjoined:1;
 	char subsup;		/* span turned into sup or sub */
 	uchar itype;		// input type =
 	uchar itype_minor;
@@ -616,8 +636,43 @@ struct htmlTag {
 #define DIS_HOVERCOLOR 5
 	uchar disval; // displayable value for the node
 	int ninp;		/* number of nonhidden inputs */
+
+/*********************************************************************
+Comments on nodeName, that is, the name of the html tag,
+or the corresponding javascript Element.
+Standard tags like p and div are case insensitive. </p> closes <P>
+p {rule} in css matches P, and so on.
+These tags become upper case in js Element.nodeName.
+t->nodeName holds the tag as written, t->nodeNameU is the upper case version.
+newTag() copies name into nodeName, then upshifts it for nodeNameU.
+These fields are not used to build dynamic tags, those made by running js.
+newTag() populates them always, but they are (primarily) used when parsing html,
+and doing the bulk match of css, before js runs.
+domlink() carries the html tag into javascript,
+and it uses nodeNameU if it is a standard, recognized tag, or nodeName
+otherwise, guessing it is a custom tag.
+But first, step back and look at the html scanner.
+It converts tags to lower case, because it's easier for me to type
+all those tags and lists of tags in lower case, and that's what tidy does too.
+Once converted, </p> closes <P>, and there is no trouble.
+The lower case versions never leave the scanner. They are a convenience.
+Conversely, the css system is upper case.
+p {rule} becomes P {rule} as a selector.
+We crosscheck this against t->nodeNameU and there is no trouble.
+Now, there is a completely separate system, createElement().
+This is javascript making tags dynamically.
+This can also take a standard tag or a custom tag.
+I convert it to low for convenience, e.g. the switch statement.
+At the end I set js nodeName to the tag name,
+or its upper case if it is a standard tag.
+The cases in switch{} duplicates the list of standard tags in html-tags.c,
+and I'm sure they don't agree, and I'm sure neither is complete.
+We need to review all this some day.
+*********************************************************************/
+
+	char *nodeName, *nodeNameU;
 // class is reserved word in c++, so use jclass for javascript class
-	char *name, *id, *jclass, *nodeName, *value, *href;
+	char *name, *id, *jclass, *value, *href;
 	const char *rvalue; /* for reset */
 	char *custom_h; // http headers for a frame or xhr tag
 	char *innerHTML; /* the html string under this tag */
@@ -635,7 +690,7 @@ enum {
 	TAGACT_NOP, TAGACT_JS, TAGACT_H, TAGACT_SUB, TAGACT_SUP, TAGACT_OVB,
 	TAGACT_OL, TAGACT_UL, TAGACT_DL, TAGACT_TEXT, TAGACT_BQ,
 	TAGACT_BODY, TAGACT_HEAD, TAGACT_DOC, TAGACT_FRAG, TAGACT_COMMENT,
-	TAGACT_MUSIC, TAGACT_SOURCE,
+	TAGACT_CDATA, TAGACT_TEMPLATE, TAGACT_MUSIC, TAGACT_SOURCE,
 	TAGACT_IMAGE, TAGACT_BR, TAGACT_IBR, TAGACT_P,
 	TAGACT_BASE, TAGACT_META, TAGACT_LINK, TAGACT_PRE, TAGACT_LABEL,
 	TAGACT_TBODY, TAGACT_THEAD, TAGACT_TFOOT,
@@ -650,6 +705,7 @@ enum {
 
 // htmlTag.itype, The order of these is important!
 enum {
+	INP_NONE,
 	INP_RESET, INP_BUTTON, INP_IMAGE, INP_SUBMIT,
 	INP_HIDDEN, INP_TEXT, INP_FILE,
 	INP_SELECT, INP_TA, INP_RADIO, INP_CHECKBOX,
