@@ -13,12 +13,13 @@ extern "C" {
 // sourcefile=buffers.c
 void setHistcontrol(void);
 void undoSpecialClear(void);
-void removeHiddenNumbers(pst p, uchar terminate);
+void removeHiddenNumbers(pst p, uchar terminate, int cx);
 pst fetchLine(int n, int show);
 void displayLine(int n);
 void printDot(void);
 void initializeReadline(void);
 pst inputLine(void);
+void freeWindowLines(struct lineMap *map);
 bool cxCompare(int cx) ;
 bool cxActive(int cx, bool error);
 bool cxQuit(int cx, int action) ;
@@ -77,6 +78,7 @@ char *htmlReformat(char *buf);
 void extractEmailAddresses(char *line);
 void cutDuplicateEmails(char *tolist, char *cclist, const char *reply);
 bool isEmailAddress(const char *s);
+bool isEmailAddressList(char *s);
 int byteOrderMark(const uchar *buf, int buflen);
 bool looksBinary(const unsigned char *buf, int buflen);
 void looks_8859_utf8(const uchar *buf, int buflen, bool * iso_p, bool * utf8_p);
@@ -100,6 +102,18 @@ char *closeColor(const char *s);
 void clearEmojis(void);
 void loadEmojis(void);
 char *selectEmoji(const char *p, int len);
+void selectLanguage(void);
+const char *i_message(int msg);
+void i_puts(int msg);
+void i_printf(int msg, ...);
+void i_printfExit(int msg, ...);
+void i_stringAndMessage(char **s, int *l, int messageNum);
+void setError(int msg, ...); //? use these only in the foreground!
+void showError(void);
+void showErrorConditional(char cmd);
+void showErrorAbort(void);
+void eb_puts(const char *s);
+bool helpUtility(void);
 
 // sourcefile=html.c
 void dwStart(void);
@@ -131,7 +145,7 @@ void delTags(int startRange, int endRange);
 void runOnload(void);
 const char *tack_fn(const char *e);
 void domSetsTimeout(int n, const char *jsrc, const char *backlink, bool isInterval);
-void scriptSetsTimeout( Tag *t);
+void scriptOnTimer( Tag *t);
 bool timerWait(int *delay_sec, int *delay_ms);
 void delTimers(const Frame *f);
 void runTimer(void);
@@ -188,6 +202,7 @@ bool reexpandFrame(void);
 
 // sourcefile=main.c
 void ebClose(int n);
+void eeCheck(void);
 void setDataSource(char *v);
 bool javaOK(const char *url);
 bool mustVerifyHost(const char *url);
@@ -210,24 +225,6 @@ bool sendMail(int account, const char **recipients, const char *body, int subjat
 bool validAccount(int n);
 bool sendMailCurrent(int sm_account, bool dosig);
 
-// sourcefile=messages.c
-void eeCheck(void);
-void eb_puts(const char *s);
-void selectLanguage(void); //@ called once at startup
-const char *i_getString(int msg); //@ the i_ functions are ok in utf8 mode
-void i_puts(int msg);
-void i_printf(int msg, ...);
-void i_printfExit(int msg, ...);
-void i_stringAndMessage(char **s, int *l, int messageNum);
-void setError(int msg, ...); //? use these only in the foreground!
-void showError(void);
-void showErrorConditional(char cmd);
-void showErrorAbort(void);
-#if 0
-void i_caseShift(unsigned char *s, char action);
-#endif
-bool helpUtility(void);
-
 // sourcefile=stringfile.c
 // Everything in this file is threadsafe except those marked with @,
 // which are still threadsafe the way we use them, e.g. only calling
@@ -254,7 +251,7 @@ void stringAndString(char **s, int *l, const char *t) ;
 void stringAndBytes(char **s, int *l, const char *t, int cnt) ;
 void stringAndChar(char **s, int *l, char c) ;
 void stringAndNum(char **s, int *l, int n) ;
-void stringAndKnum(char **s, int *l, int n) ;
+#define stringAndMessage(s, l, m) stringAndString(s, l, i_message(m))
 char *cloneString(const char *s) ;
 char *cloneMemory(const char *s, int n) ;
 void leftClipString(char *s) ;
@@ -328,7 +325,7 @@ char *makeAbsPath(const char *f);
 const char *nextScanFile(const char *base);
 bool readDirectory(const char *filename, int endline, char cmd, struct lineMap **map_p);
 bool delFiles(int start, int end, bool withtext, char origcmd, char *cmd_p);
-bool moveFiles(int start, int end, int dest, char origcmd);
+bool moveFiles(int start, int end, int dest, char origcmd, char relative);
 bool envFile(const char *line, const char **expanded); //@
 char *envFileAlloc(const char *line); //@
 bool envFileDown(const char *line, const char **expanded) ; //@
@@ -373,7 +370,7 @@ bool frameSecurityFile(const char *thisfile);
 bool receiveCookie(const char *url, const char *str) ;
 void cookiesFromJar(void) ;
 bool isInDomain(const char *d, const char *s);
-void sendCookies(char **s, int *l, const char *url, bool issecure) ;
+void findcookies(char **s, int *l, const char *url, bool issecure) ;
 void mergeCookies(void);
 void setupEdbrowseCache(void);
 void clearCache(void) ;
@@ -394,12 +391,14 @@ void ircSetFileName(Window *w);
 bool ircWrite(void);
 void ircRead(void);
 bool ircSetup(char *line);
+void ircClose(Window *w);
 void ircReadlineControl(void);
 void ircReadlineRelease(void);
 
 // sourcefile=css.c
 void writeShortCache(void);
 bool matchMedia(char *t);
+Frame *frameFromWindow(int gsn);
 void cssDocLoad(int frameNumber, char *s, bool pageload);
 void cssFree(Frame *f);
 Tag **querySelectorAll(const char *selstring, Tag *top);
@@ -429,6 +428,8 @@ int run_function_onearg_t(const Tag *t, const char *name, const Tag *t2);
 int run_function_onearg_win(const Frame *f, const char *name, const Tag *t2);
 int run_function_onearg_doc(const Frame *f, const char *name, const Tag *t2);
 void run_function_onestring_t(const Tag *t, const char *name, const char *s);
+char *run_function_onestring1_t(const Tag *t, const char *name, const char *s);
+void run_function_twostring_t(const Tag *t, const char *name, const char *s1, const char *s2);
 void run_function_onestring_win(const Frame *f, const char *name, const char *s);
 void jsRunData(const Tag *t, const char *filename, int lineno);
 bool run_event_t(const Tag *t, const char *pname, const char *evname);
@@ -436,6 +437,7 @@ bool run_event_win(const Frame *f, const char *pname, const char *evname);
 bool run_event_doc(const Frame *f, const char *pname, const char *evname);
 bool bubble_event_t(const Tag *t, const char *name);
 void set_property_bool_win(const Frame *f, const char *name, bool v);
+void set_property_bool_doc(const Frame *f, const char *name, bool v);
 char *get_property_url_t(const Tag *t, bool action);
 char *get_style_string_t(const Tag *t, const char *name);
 void delete_property_t(const Tag *t, const char *name);
@@ -445,21 +447,18 @@ bool get_property_bool_t(const Tag *t, const char *name);
 enum ej_proptype typeof_property_t(const Tag *t, const char *name);
 int get_property_number_t(const Tag *t, const char *name);
 char * get_property_string_t(const Tag *t, const char *name);
-char *get_dataset_string_t(const Tag *t, const char *name);
 void set_property_bool_t(const Tag *t, const char *name, bool v);
 void set_property_number_t(const Tag *t, const char *name, int v);
 void set_property_string_t(const Tag *t, const char *name, const char * v);
-void set_dataset_string_t(const Tag *t, const char *name, const char * v);
-void set_property_bool_win(const Frame *f, const char *name, bool v);
 void set_property_string_win(const Frame *f, const char *name, const char *v);
 void set_property_object_doc(const Frame *f, const char *name, const Tag *t2);
 void set_property_string_doc(const Frame *f, const char *name, const char *v);
 void jsRunScriptWin(const char *str, const char *filename, 		 int lineno);
 void jsRunScript_t(const Tag *t, const char *str, const char *filename, 		 int lineno);
 char *jsRunScriptWinResult(const char *str, const char *filename, int lineno) ;
-void establish_js_option(Tag *t, Tag *sel);
+void establish_js_option(Tag *t, Tag *sel, Tag *og);
 void establish_js_textnode(Tag *t, const char *fpn);
-void domLink(Tag *t, const char *classname, const char *href, const char *list, const Tag *owntag, int extra);
+void domLink(Tag *t, const char *classname, const char *list, const Tag *owntag, int extra);
 void rebuildSelectors(void);
 bool has_gcs(const char *name);
 enum ej_proptype typeof_gcs(const char *name);
