@@ -169,7 +169,7 @@ bool setEnvelopeFormat(const char *s)
 
 // check
 	for(j=0; (c = s[j]); ++j) {
-		if(isspace(c))
+		if(isspaceByte(c))
 			continue;
 		something = true;
 		if(!strchr(envelopeFormatChars, c))
@@ -184,7 +184,7 @@ bool setEnvelopeFormat(const char *s)
 
 	memset(count, 0, sizeof(count));
 	for(i=0; (c = *s); ++s) {
-		if(isspace(c))
+		if(isspaceByte(c))
 			continue;
 		j = strchr(envelopeFormatChars, c) - envelopeFormatChars;
 		if(count[j])
@@ -202,7 +202,7 @@ static void setLimit(const char *t)
 	skipWhite(&t);
 	if(*t == '-')
 		early = true, ++t;
-	if (!isdigit(*t)) {
+	if (!isdigitByte(*t)) {
 		i_puts(MSG_NumberExpected);
 		return;
 	}
@@ -473,8 +473,8 @@ static bool imapSearch(CURL * handle, struct FOLDER *f, char *line)
 	if (*line && line[1] == ' ' && strchr("sfb", *line)) {
 		searchtype = *line;
 		line += 2;
-	} else if (line[0] && !isspace(line[0]) &&
-		   (isspace(line[1]) || !line[1])) {
+	} else if (line[0] && !isspaceByte(line[0]) &&
+		   (isspaceByte(line[1]) || !line[1])) {
 		i_puts(MSG_ImapSearchHelp);
 		return false;
 	}
@@ -523,10 +523,10 @@ none:
 		++t;
 	u = t;
 	while (*u) {
-		if (!isdigit(*u))
+		if (!isdigitByte(*u))
 			break;
 		++cnt;
-		while (isdigit(*u))
+		while (isdigitByte(*u))
 			++u;
 		skipWhite2(&u);
 	}
@@ -613,18 +613,27 @@ static void viewAll(struct FOLDER *f)
 	}
 }
 
-// 0 ok, -1 curl error, 1 something was deleted
-static int bulkMoveDelete(CURL * handle, struct FOLDER *f,
+static bool expunge(CURL * handle)
+{
+	CURLcode res;
+	curl_easy_setopt(handle, CURLOPT_CUSTOMREQUEST, "EXPUNGE");
+	mailstring = initString(&mailstring_l);
+	res = getMailData(handle);
+	nzFree(mailstring);
+	return res == CURLE_OK;
+}
+
+static bool bulkMoveDelete(CURL * handle, struct FOLDER *f,
 			  struct MIF *this_mif, char key, char subkey,
 			  struct FOLDER *destination)
 {
-	CURLcode res = CURLE_OK;
+	CURLcode res;
 	char cust_cmd[80];
 	int j;
 	struct MIF *mif = f->mlist;
-	int mcnt = 0;		// move count
+	int ex_cnt = 0;		// expunge count
 	char *t, *fromline = 0;
-	bool yesdel = false;
+	bool marked_for_delete = false;
 
 	if (key == 'f') {
 		fromline = this_mif->from;
@@ -638,7 +647,7 @@ static int bulkMoveDelete(CURL * handle, struct FOLDER *f,
 		bool delflag = false;
 		if (mif->gone)
 			continue;
-		mif->seqno -= mcnt;
+		mif->seqno -= ex_cnt;
 		if (fromline && !stringEqual(fromline, mif->from))
 			continue;
 		if (subkey == 'm') {
@@ -655,11 +664,13 @@ static int bulkMoveDelete(CURL * handle, struct FOLDER *f,
 // The move command shifts all the sequence numbers down.
 			if (move_capable) {
 				mif->gone = true;
-				++mcnt;
+				++ex_cnt;
 			} else {
+// we did copy not move, so delete
 				delflag = true;
 			}
 		}
+
 		if (subkey == 'd' || delflag) {
 			sprintf(cust_cmd, "STORE %d +Flags \\Deleted",
 				mif->seqno);
@@ -670,14 +681,80 @@ static int bulkMoveDelete(CURL * handle, struct FOLDER *f,
 			if (res != CURLE_OK)
 				goto abort;
 			mif->gone = true;
-			yesdel = true;
+			marked_for_delete = true;
+
+/*********************************************************************
+We have thus far marked thismessage for delete, but it's not really gone
+until we expunge. In contrast, moving a message always expunges it.
+It is after all somewhere else.
+Expunge means the message is not here, and the server resequences the numbers,
+and we have to do the same in our representation of the folder.
+If we don't stay in sync with the server, the next command could read or move
+or delete the wrong message. Ouch!
+ex_cnt is an expunge count, how many messages expunged,
+how far should we shift our sequence numbers to stay in sync with the server?
+At this point, this message is marked for deletion but not expunged,
+don't increment ex_cnt.
+At the end of the folder we can command the server to expunge,
+and the deleted messages will all go away.
+Fine, but, if you issue move and delete commands in one pass,
+imap servers behave in very different ways.
+Sometimes the move command expunges messages that were marked for deletion,
+along with the message being moved.
+Sometimes it doesn't. And sometimes it depends
+on whether the deleted message comes before or after the moved message.
+And sometimes it depends on the folder.
+We can't possibly predict the behavior of the server,
+and adjust our sequence numbers accordingly.
+There are two ways around this: the right way and the easy way.
+We can watch the curl traffic and look for
+* n EXPUNGE
+and each time we see that, decrement the sequence numbers beyond n,
+for message n is gone.
+Or, we can expunge after every delete. That forces the server
+to behave in a predictable manner, whence we can follow along.
+And that's what I do.
+Unfortunately, d is now an irreversible command.
+Type d and the message is gone.
+You can't say "oops I didn't mean to delete that" and q to quit
+and log back in and it will still be there.
+A hybrid solution might be to remember the deletes and then expunge at end of
+folder or when a move command is issued.
+Now d is reversible again.
+I may implement that some day, but it's not as easy as it seems.
+On some servers move is actually copy + delete, and in that case
+we want to delete + expunge.
+So 'd' means mark for delete; delete implied from 'm' means delete + expunge.
+I haven't worked all this out, so for now let's just expunge every time.
+A hybrid solution wouldn't be implemented here anyways,
+this is bulk move or bulk delete.
+Bear in mind, d irreversible is consistent with the pop3 interface,
+so maybe it's not such a bad thihng.
+Now - all that said, we don't have to do the delete expunge here,
+because it isn't a mix of move and delete commands.
+It is either bulk move or bulk delete.
+We may have deleted or moved before this bulk command was issued,
+but thence we are in sync with the server.
+So just mark for delete and expunge at the end,
+like we did before.
+*********************************************************************/
+
+#if 0
+			debugPrint(3, "` %d EXPUNGE", mif->seqno);
+			if(!expunge(handle)) goto abort;
+			++ex_cnt;
+#endif
 		}
 	}
 
-	return yesdel;
+	if(marked_for_delete) {
+		debugPrint(3, "` EXPUNGE");
+		if(!expunge(handle)) goto abort;
+	}
+	return true;
 
 abort:
-	return -1;
+	return false;
 }
 
 
@@ -685,14 +762,14 @@ abort:
 static char postkey;
 static void scanFolder(CURL * handle, struct FOLDER *f)
 {
-	struct MIF *mif;
-	int j, rc;
+	struct MIF *mif, *mif2;
+	int j, j2, rc;
 	CURLcode res = CURLE_OK;
 	char *t;
 	char key;
 	char cust_cmd[80];
 	char inputline[80];
-	bool yesdel = false, delflag;
+	bool delflag;
 
 	if (!f->nmsgs) {
 		i_puts(MSG_NoMessages);
@@ -790,10 +867,10 @@ You'll see this after the perform function runs.
 			curl_easy_setopt(handle, CURLOPT_HEADERFUNCTION, NULL);
 			undosOneMessage();
 			if (res != CURLE_OK) {
-				ebcurl_setError(res, mailbox_url, 1,
-						emptyString);
+// I don't know why the read message didn't work, probably connection error,
+// curl is probably going to log back in, so abort this folder.
 				nzFree(mailstring);
-				goto action;
+				goto abort;
 			}
 
 /* have to strip 2 fetch BODY lines off the front,
@@ -825,7 +902,7 @@ You'll see this after the perform function runs.
 		}
 
 		if (key == 'p') {
-			int j2 = j;
+			j2 = j;
 			while(--j >= 0) {
 				--mif;
 				if(!mif->gone)
@@ -874,11 +951,8 @@ You'll see this after the perform function runs.
 					goto reaction;
 				}
 			}
-			rc = bulkMoveDelete(handle, f, mif, key, subkey, g);
-			if (rc < 0)
+			if(!bulkMoveDelete(handle, f, mif, key, subkey, g))
 				goto abort;
-			if (rc)
-				yesdel = true;
 // If current mail is gone, step ahead.
 			if (mif->gone)
 				continue;
@@ -930,8 +1004,8 @@ You'll see this after the perform function runs.
 // The move command shifts all the sequence numbers down.
 				if (move_capable) {
 					mif->gone = true;
-					int j2 = j + 1;
-					struct MIF *mif2 = mif + 1;
+					j2 = j + 1;
+					mif2 = mif + 1;
 					while (j2 < f->nfetch) {
 						--mif2->seqno;
 						++j2, ++mif2;
@@ -956,17 +1030,14 @@ You'll see this after the perform function runs.
 		if (res != CURLE_OK)
 			goto abort;
 		mif->gone = true;
-		yesdel = true;
-	}
-
-	if (yesdel) {
-/* things deleted, time to expunge */
-		curl_easy_setopt(handle, CURLOPT_CUSTOMREQUEST, "EXPUNGE");
-		mailstring = initString(&mailstring_l);
-		res = getMailData(handle);
-		nzFree(mailstring);
-		if (res != CURLE_OK)
-			goto abort;
+		debugPrint(3, "` %d EXPUNGE", mif->seqno);
+		if(!expunge(handle)) goto abort;
+		j2 = j + 1;
+		mif2 = mif + 1;
+		while (j2 < f->nfetch) {
+			--mif2->seqno;
+			++j2, ++mif2;
+		}
 	}
 
 	puts("end of folder");
@@ -1000,15 +1071,25 @@ char **name, char **addr)
 	char *t = *t0, *u;
 	if(!strncmp(t, "NIL ", 4)) {
 		t += 4;
+	} else if(*t == '{') {
+		 int l = strtol(t+1, &t, 10);
+		if(*t == '}') ++t;
+		// with number in braces, subject is on next line.
+// isspace takes us past crlf
+		while(isspaceByte(*t)) ++t;
+		u = t + l;
+		*u = 0;
+// allow for quotes either side
+		if(*t == '"' && u[-1] == '"') ++t, u[-1] = 0;
+		if(name) *name = t;
+		*t0 = t = u + 2;
 	} else {
-		if(*t != '"')
-			return false;
+		if(*t != '"') return false;
 		u = nextRealQuote(++t);
 		if (!u)
 			return false;
 		*u = 0;
-		if(name)
-			*name = t;
+		if(name) *name = t;
 		*t0 = t = u + 2;
 	}
 // next should come NIL "email" "domain"
@@ -1056,7 +1137,7 @@ static void envelopes(CURL * handle, struct FOLDER *f)
 			t += 11;
 			while (*t == ' ')
 				++t;
-			if (isdigit(*t))
+			if (isdigitByte(*t))
 				mif->uid = atoi(t);
 		}
 		nzFree(mailstring);
@@ -1113,14 +1194,32 @@ We'll deal with that later.
 		mif->prec = emptyString;
 		mif->ccrec = emptyString;
 
+/*********************************************************************
+Before we start cracking, here are a couple of illustrative examples.
+You can get these, or another envelope that doesn't seem to parse
+correctly, by db4. Note in the second example,
+that the encoded subject is 105 bytes, which matches {105}
+* 358 FETCH (FLAGS (\Seen $HasAttachment) INTERNALDATE "03-Dec-2022 12:57:23 +0000" RFC822.SIZE 121285 ENVELOPE ("Sat, 03 Dec 2022 12:57:12 +0000" "Your Daily Digest for Sat, Dec 03" ((NIL NIL "USPSInformeddelivery" "email.informeddelivery.usps.com")) ((NIL NIL "USPSInformeddelivery" "email.informeddelivery.usps.com")) ((NIL NIL "USPSInformeddelivery" "email.informeddelivery.usps.com")) ((NIL NIL "KWNRE" "comcast.net")) NIL NIL NIL "<20221203125712.0a463f28d45908a3@email.informeddelivery.usps.com>"))
+* 359 FETCH (FLAGS ($HasNoAttachment) INTERNALDATE "03-Dec-2022 16:06:55 +0000" RFC822.SIZE 33094 ENVELOPE ("3 Dec 2022 16:01:33 +0000" {105}
+=?Windows-1252?Q?Wendy=2C_very_important=2E_=28Note=3A_Failur?= =?Windows-1252?Q?e_to_act_in_time=85=29?= (("Publishers Clearing House" NIL "PublishersClearingHouse" "e.superprize.pch.com")) (("Publishers Clearing House" NIL "PublishersClearingHouse" "e.superprize.pch.com")) (("Publishers Clearing House" NIL "PublishersClearingHouse" "e.superprize.pch.com")) (("Wendy Dahlke" NIL "kwnre" "comcast.net")) NIL NIL NIL "<543.30570379008.202212031601292260087.0123746943@e.superprize.pch.com>"))
+* 360 FETCH (FLAGS ($HasNoAttachment) INTERNALDATE "04-Dec-2022 06:27:36 +0000" RFC822.SIZE 7083 ENVELOPE ("Sun, 4 Dec 2022 06:27:35 +0000" "Discover The Power Of Comparison Shopping" (({21}
+Plan Offers, Medicare NIL "deck" "uniqueencrypter.com")) (({21}
+Plan Offers, Medicare NIL "deck" "uniqueencrypter.com")) (({21}
+Plan Offers, Medicare NIL "deck" "uniqueencrypter.com")) ((NIL NIL "kwnre" "comcast.net")) NIL NIL NIL "<01000184dbd191f0-a0febe67-5ffe-4875-b066-8bf71e7bcedb-000000@email.amazonses.com>"))
+* 361 FETCH (FLAGS ($HasNoAttachment) INTERNALDATE "04-Dec-2022 08:26:13 +0000" RFC822.SIZE 5968 ENVELOPE ("Sun, 4 Dec 2022 08:26:11 +0000" "=?UTF-8?B?4biMbyB5b3Ugd2FrZSB1cCDhua1pcmVkLCB3aXRoIGRyeSBza2luICYgIOG2gWFtYWdlZCBoYWlyPw==?=" (({20}
+"=?UTF-8?Q?BLlSSY?=" NIL "WeCare4U" "cateringadvetiser.com")) (({20}
+"=?UTF-8?Q?BLlSSY?=" NIL "WeCare4U" "cateringadvetiser.com")) (({20}
+"=?UTF-8?Q?BLlSSY?=" NIL "WeCare4U" "cateringadvetiser.com")) ((NIL NIL "kwnre" "comcast.net")) NIL NIL NIL "<01010184dc3e266d-c67755c2-facc-4650-9375-f85825069dd6-000000@us-west-2.amazonses.com>"))
+*********************************************************************/
+
 		t = strstr(mailstring, "ENVELOPE (");
 		if (!t) {
 			nzFree(mailstring);
 			continue;
 		}
 
-/* pull out subject, reply, etc */
-/* Don't free mailstring, we're using pieces of it */
+// pull out subject, reply, etc
+// Don't free mailstring, we're using pieces of it
 		t += 10;
 		while (*t == ' ')
 			++t;
@@ -1139,7 +1238,7 @@ We'll deal with that later.
 			++t;
 	}
 
-/* subject next, I'll assume it is always quoted */
+// subject next, I'll assume it is always quoted
 		while (*t == ' ')
 			++t;
 
@@ -1152,12 +1251,10 @@ We'll deal with that later.
 		sublength = -1;
 		if(*t == '{') {
 			 sublength = strtol(t+1, &t, 10);
-			if(*t == '}')
-				++t;
+			if(*t == '}') ++t;
 		// with number in braces, subject is on next line.
 // isspace takes us past crlf
-			while(isspace(*t))
-				++t;
+			while(isspaceByte(*t)) ++t;
 		}
 
 //  printf("%d,%d,%d|%s|\n", mailstring_l, t-mailstring, strlen(t), t);
@@ -1183,7 +1280,7 @@ We'll deal with that later.
 doreply:
 		while (*t == ' ')
 			++t;
-		if (strncmp(t, "((\"", 3))
+		if (t[0] != '(' || t[1] != '(')
 			goto doref;
 		t += 2;
 		if (!grabEmailFromEnvelope(&t, &mif->from, &mif->reply))
@@ -1191,6 +1288,7 @@ doreply:
 
 // We have parsed from-reply in block 1, block 4 contains the recipients,
 // Don't know what is in blocks 2 and 3.
+// shouldn't we be checking for (( or NIL here?
 		u = strstr(t, "(("); // block 2
 		if(!u)
 			goto doref;
@@ -1273,7 +1371,7 @@ dosize:
 		if (!u)
 			continue;
 		t = u + 5;
-		if (!isdigit(*t))
+		if (!isdigitByte(*t))
 			continue;
 		mif->size = atoi(t);
 
@@ -1303,8 +1401,8 @@ static void examineFolder(CURL * handle, struct FOLDER *f, bool dostats)
 	if (t) {
 		while (*t == ' ')
 			--t;
-		if (isdigit(*t)) {
-			while (isdigit(*t))
+		if (isdigitByte(*t)) {
+			while (isdigitByte(*t))
 				--t;
 			++t;
 			f->nmsgs = atoi(t);
@@ -1322,7 +1420,7 @@ static void examineFolder(CURL * handle, struct FOLDER *f, bool dostats)
 		t += 8;
 		while (*t == ' ')
 			++t;
-		if (isdigit(*t))
+		if (isdigitByte(*t))
 			f->uidnext = atoi(t);
 	}
 
@@ -1534,14 +1632,14 @@ refresh:
 		}
 
 		t = inputline;
-		if (t[0] == 'd' && t[1] == 'b' && isdigit(t[2]) && !t[3]) {
+		if (t[0] == 'd' && t[1] == 'b' && isdigitByte(t[2]) && !t[3]) {
 			debugLevel = t[2] - '0';
 			curl_easy_setopt(handle, CURLOPT_VERBOSE,
 					 (debugLevel >= 4));
 			goto input;
 		}
 
-		if (*t == 'l' && isspace(t[1])) {
+		if (*t == 'l' && isspaceByte(t[1])) {
 			setLimit(t + 1);
 			goto input;
 		}
@@ -1551,7 +1649,7 @@ refresh:
 				setEnvelopeFormat("");
 				goto input;
 			}
-			if(isspace(t[1]) && setEnvelopeFormat(t + 2))
+			if(isspaceByte(t[1]) && setEnvelopeFormat(t + 2))
 				goto input;
 		}
 
@@ -1635,7 +1733,7 @@ refresh:
 			last_nl = true;
 			continue;
 		}
-		if (last_nl && isdigit(mailstring[i]))
+		if (last_nl && isdigitByte(mailstring[i]))
 			num_messages++;
 		last_nl = false;
 	}
@@ -1957,7 +2055,7 @@ key_command:
 
 // At this point we're saving the mail somewhere.
 writeMail:
-	if (!isimap || isupper(key))
+	if (!isimap || isupperByte(key))
 		delflag = true;
 	atname = 0;
 	if (!isimap)
@@ -2391,6 +2489,15 @@ copy:
 	if (len)
 		memmove(t, s, len);
 	vr = t + len;
+// remove spaces between blocks
+	for(t = start; t < vr; ++t) {
+		if(!isspaceByte(*t)) break;
+	}
+	if(t > start && (t == vr || (t[0] == '=' && t[1] == '?'))) {
+		len = vr - t;
+		memmove(start, t, len);
+		vr = start + len;
+	}
 	goto restart;
 
 finish:
