@@ -1,8 +1,14 @@
-/* format.c, Format text, establish line breaks, manage whitespace,
-iso8859, utf8, utf16, utf32, base64, color codes, emojis. */
+// format.c, Format text, establish line breaks, manage whitespace,
+// iso8859, utf8, utf16, utf32, base64, color codes,
+// emojis, messages in your local language.
 
 #include "eb.h"
 
+// It's possible that console has to be utf8, or edbrowse won't work properly.
+bool cons_utf8, iuConvert = true;
+char type8859 = 1;
+bool helpMessagesOn;
+
 /*********************************************************************
 Prepare html for text processing.
 Change nulls to spaces.
@@ -84,7 +90,7 @@ respace:
 // tidy turns <td> <i> hello </i> </td> into <td><i>hello</i></td>
 // But not so with <p> or other strong tags.
 // I try to do the same.
-		if(s[1] != InternalCodeChar || !isdigit(s[2])) continue;
+		if(s[1] != InternalCodeChar || !isdigitByte(s[2])) continue;
 			n = strtol(s + 2, &u, 10);
 // leave input fields alone
 			if(*u != '*' && *u != '{') continue;
@@ -289,13 +295,13 @@ static void anchorSwap(char *buf)
 		change = false;
 		++cnt;
 		premode = false;
-/* w represents the state of whitespace */
+// w represents the state of whitespace
 		w = NULL;
-/* a points to the prior anchor, which is swappable with following whitespace */
+// a points to the prior anchor, which is swappable with following whitespace
 		a = NULL;
 
 		for (s = buf; (c = *s); ++s) {
-			if (isspaceByte(c) || c == '|') {
+			if (isspaceByte(c) || (c == '|' && !premode)) {
 				if (c == '\t' && !premode)
 					*s = ' ';
 				if (!w)
@@ -465,7 +471,7 @@ static void html_ws(char *buf)
 		strong = false;
 // watch for pipes inside whitespace, these are usually cell delimiters.
 // pipe ends a whitespace region if only spaces come before
-		for (w = s; isspaceByte(*w) || *w == '|'; ++w) {
+		for (w = s; isspaceByte(*w) || (*w == '|' && !premode); ++w) {
 			if(*w == '|') {
 				if(strong) continue;
 				break;
@@ -1179,7 +1185,7 @@ char *htmlReformat(char *buf)
 /* It's a little thing really, but the blank line at the top of each frame annoys me */
 	fmark = new;
 	while ((fmark = strstr(fmark + 1, "*`--\n\n"))) {
-		if (isdigit(fmark[-1]))
+		if (isdigitByte(fmark[-1]))
 			strmove(fmark + 5, fmark + 6);
 	}
 
@@ -1327,7 +1333,7 @@ bool isEmailAddress(const char *s)
 		if (c&0x80)	// nonascii
 			return false;
 		if (atfound) {
-			if (!isalnum(c) && c != '.' && c != '-')
+			if (!isalnumByte(c) && c != '.' && c != '-')
 				return false;
 			if (c == '.') {
 				if (s[1] == '.' || s[1] == 0 || s[-1] == '.'
@@ -1346,6 +1352,22 @@ bool isEmailAddress(const char *s)
 	return atfound & dotfound;
 }
 
+/* Check for a list of email addresses separated by commas. */
+bool isEmailAddressList(char *s)
+{
+	if (!s || !*s)
+		return false;
+	char *e = strchr(s, ',');
+	if (!e) return isEmailAddress(s);
+	if (isEmailAddressList(e + 1)) {
+		*e = 0;
+		bool starts_well = isEmailAddress(s);
+		*e = ',';
+		return starts_well;
+	}
+	return false;
+}
+
 /* return 1 for utf16, 2 for utf32, ored with 4 for big endian */
 int byteOrderMark(const uchar * buf, int buflen)
 {
@@ -1939,7 +1961,6 @@ void utfLow(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p,
 // rbuf is passed by reference because some of the conversion routines
 // create a new allocated array which must be passed back.
 // This is called by readFile in buffers.c, so has some
-// confusing legacy interactions with the global variable serverData.
 void diagnoseAndConvert (char **rbuf_p, bool *isAllocated_p, int *partSize_p, const bool firstPart, const bool showMessage)
 {
 	char *rbuf = *rbuf_p;
@@ -1970,7 +1991,6 @@ void diagnoseAndConvert (char **rbuf_p, bool *isAllocated_p, int *partSize_p, co
 		rbuf[j] = 0;
 		fileSize -= (*partSize_p - j);
 		*partSize_p = j;
-		serverDataLen = fileSize;
 	}
 
 	if (!iuConvert) return;
@@ -1989,8 +2009,8 @@ void diagnoseAndConvert (char **rbuf_p, bool *isAllocated_p, int *partSize_p, co
 		utfLow(rbuf, *partSize_p, &tbuf, &*partSize_p, bom);
 		if(*isAllocated_p) nzFree(rbuf);
 		*isAllocated_p = true;
-		serverData = *rbuf_p = rbuf = tbuf;
-		serverDataLen = fileSize = *partSize_p;
+		*rbuf_p = rbuf = tbuf;
+		fileSize = *partSize_p;
 	} else {
 		int oldSize = *partSize_p;
 		looks_8859_utf8((uchar *) rbuf, *partSize_p,
@@ -2007,9 +2027,8 @@ void diagnoseAndConvert (char **rbuf_p, bool *isAllocated_p, int *partSize_p, co
 				(uchar **) & tbuf, &*partSize_p);
 			if(*isAllocated_p) nzFree(rbuf);
 			*isAllocated_p = true;
-			serverData = *rbuf_p = rbuf = tbuf;
+			*rbuf_p = rbuf = tbuf;
 			fileSize += (*partSize_p - oldSize);
-			serverDataLen = fileSize;
 		}
 		if (!cons_utf8 && isutf8) {
 			if ((debugLevel >= 2 || (debugLevel == 1 && showMessage))
@@ -2019,9 +2038,8 @@ void diagnoseAndConvert (char **rbuf_p, bool *isAllocated_p, int *partSize_p, co
 				(uchar **) & tbuf, &*partSize_p);
 			if(*isAllocated_p) nzFree(rbuf);
 			*isAllocated_p = true;
-			serverData = *rbuf_p = rbuf = tbuf;
+			*rbuf_p = rbuf = tbuf;
 			fileSize += (*partSize_p - oldSize);
-			serverDataLen = fileSize;
 		}
 		if (cons_utf8 && isutf8 && firstPart) {
 // Strip off the leading bom, if any, and no we're not going to put it back.
@@ -2029,7 +2047,6 @@ void diagnoseAndConvert (char **rbuf_p, bool *isAllocated_p, int *partSize_p, co
 			    !memcmp(rbuf, "\xef\xbb\xbf", 3)) {
 				fileSize -= 3, *partSize_p -= 3;
 				memmove(rbuf, rbuf + 3, *partSize_p);
-				serverDataLen = *partSize_p;
 			}
 		}
 	}
@@ -2502,21 +2519,21 @@ char *closeColor(const char *s)
 
 	if (!strncmp(s, "rgb(", 4)) {
 		t = s + 4;
-		if (!isdigit(*t))
+		if (!isdigitByte(*t))
 			goto fail;
 		r1 = strtol(t, (char **)&t, 10);
 		if (*t == ',')
 			++t;
 		while (*t == ' ')
 			++t;
-		if (!isdigit(*t))
+		if (!isdigitByte(*t))
 			goto fail;
 		g1 = strtol(t, (char **)&t, 10);
 		if (*t == ',')
 			++t;
 		while (*t == ' ')
 			++t;
-		if (!isdigit(*t))
+		if (!isdigitByte(*t))
 			goto fail;
 		b1 = strtol(t, (char **)&t, 10);
 		if (*t == ',')
@@ -2541,7 +2558,7 @@ char *closeColor(const char *s)
 	} else {
 // not an rgb format we recognize; should be just a word.
 		for (t = s; *t; ++t)
-			if (!isalpha(*t))
+			if (!isalphaByte(*t))
 				goto fail;
 		return (char *)s;
 	}
@@ -2654,7 +2671,7 @@ void loadEmojis(void)
 			i_printf(MSG_EmojiBadControl, c, lineno);
 			goto fail;
 		}
-		if((signed char)c > ' ' && !isalnum(c) && c != '{' && c != '}') {
+		if((signed char)c > ' ' && !isalnumByte(c) && c != '{' && c != '}') {
 			i_printf(MSG_EmojiBadChar, c, lineno);
 			goto fail;
 		}
@@ -2674,7 +2691,7 @@ void loadEmojis(void)
 		}
 		if(!g) {
 // need to start a new group
-			for(u = s; isalnum(*u); ++u)  ;
+			for(u = s; isalnumByte(*u); ++u)  ;
 			if(u == s || !stringEqual(u, " {")) {
 				i_printf(MSG_EmojiSyntax, lineno);
 				s = t; continue;
@@ -2796,7 +2813,7 @@ tty_complete:
 	inInput = false;
 	state = 2, ejcommas = 0;
 	for(i = j = 0; (c = ejline[i]); ++i) {
-		if(isspace(c)) c = ' ';
+		if(isspaceByte(c)) c = ' ';
 		if(c == ' ') {
 			if(!state) ejline[j++] = c, state = 1;
 			continue;
@@ -2957,3 +2974,296 @@ top:
 	return ejresponse;
 }
 
+// Messages in your host language.
+static const char **messageArray;
+char eb_language[9];
+int eb_lang;
+const char *const supported_languages[] = { 0,
+	"english", "french", "portuguese", "polish",
+	"german", "russian", "italian", "spanish",
+};
+// don't forget allMonths in stringfile.c
+
+// startup .ebrc files in various languages
+const char *ebrc_string;
+static const char *qrg_string;
+
+#include <locale.h>
+
+void selectLanguage(void)
+{
+	char *s = getenv("LANG");
+	char *dot;
+
+// default is English
+	strcpy(eb_language, "en");
+	eb_lang = 1;
+	messageArray = msg_en;
+	ebrc_string = ebrc_en;
+	qrg_string = qrg_en;
+
+	if (!s || !*s) return;
+
+	if (strcasestr(s, "utf8") || strcasestr(s, "utf-8"))
+		cons_utf8 = true;
+
+/*********************************************************************
+We roll our own international messages in this file, so you wouldn't think
+we need setlocale, but pcre needs the locale for expressions like \w,
+and for ranges like [A-z],
+and to convert to upper or lower case etc.
+So I set LC_ALL, which covers both LC_CTYPE and LC_COLLATE.
+By calling strcoll, the directory scan is in the same order as ls.
+See dircmp_alph() in stringfile.c
+*********************************************************************/
+	setlocale(LC_ALL, "");
+
+/* But LC_TIME controls time/date formatting, I.E., strftime.  The one
+ * place we do that, we need standard day/month abbreviations, not
+ * localized ones.  So LC_TIME needs to be C. */
+	setlocale(LC_TIME, "C");
+
+	strncpy(eb_language, s, 7);
+	eb_language[7] = 0;
+	caseShift(eb_language, 'l');
+	dot = strchr(eb_language, '.');
+	if (dot)
+		*dot = 0;
+
+	if (!strncmp(eb_language, "en", 2))
+		return;		// english is already default
+
+	if (!strncmp(eb_language, "fr", 2)) {
+		eb_lang = 2;
+		messageArray = msg_fr;
+		ebrc_string = ebrc_fr;
+		qrg_string = qrg_fr;
+		type8859 = 1;
+		return;
+	}
+
+	if (!strncmp(eb_language, "pt", 2)) {
+// This is Brazillian Portuguese. We use to key on pt_br.
+// It is close to pt_pt but not identical. I assume it is close enough.
+// Someone from Portugal can write msg_pt_pt if they wish.
+		eb_lang = 3;
+		messageArray = msg_pt_br;
+		ebrc_string = ebrc_pt_br;
+		qrg_string = qrg_pt_br;
+		type8859 = 1;
+		return;
+	}
+
+	if (!strncmp(eb_language, "pl", 2)) {
+		eb_lang = 4;
+		messageArray = msg_pl;
+		ebrc_string = ebrc_pl;
+		type8859 = 2;
+		return;
+	}
+
+	if (!strncmp(eb_language, "de", 2)) {
+		eb_lang = 5;
+		messageArray = msg_de;
+		ebrc_string = ebrc_de;
+		type8859 = 1;
+		return;
+	}
+
+	if (!strncmp(eb_language, "ru", 2)) {
+		eb_lang = 6;
+		messageArray = msg_ru;
+		ebrc_string = ebrc_ru;
+		type8859 = 5;
+		return;
+	}
+
+	if (!strncmp(eb_language, "it", 2)) {
+		eb_lang = 7;
+		messageArray = msg_it;
+		type8859 = 1;
+		return;
+	}
+
+	if (!strncmp(eb_language, "es", 2)) {
+		eb_lang = 8;
+		messageArray = msg_es;
+		type8859 = 1;
+		ebrc_string = ebrc_es;
+		qrg_string = qrg_es;
+		return;
+	}
+
+/* This error is really annoying if it pops up every time you invoke edbrowse.
+	fprintf(stderr, "Sorry, language %s is not implemented\n", buf);
+*/
+}
+
+/*********************************************************************
+WARNING: this routine, which is at the heart of the international prints
+i_puts i_printf, is not threadsafe in iso8859 mode.
+Well utf8 has been the default console standard since er um 2000,
+and I'm almost ready to chuck iso8859 console support altogether.
+*********************************************************************/
+
+const char *i_message(int msg)
+{
+	const char **a = messageArray;
+	const char *s;
+	char *t;
+	int t_len;
+// large enough for any edbrowse message
+	static char utfbuf[1000];
+	if (msg >= EdbrowseMessageCount) s = emptyString;
+	else s = a[msg];
+	if (!s) s = msg_en[msg];
+	if (!s) s = "spurious message";
+	if (cons_utf8) return s;
+
+// Oops, we have to convert
+	utf2iso((uchar *) s, strlen(s), (uchar **) & t, &t_len);
+	strcpy(utfbuf, t);
+	nzFree(t);
+	return utfbuf;
+}
+
+/*********************************************************************
+Internationalize the standard puts and printf.
+These are simple informational messages, where you don't need to error out,
+or check the debug level, or store the error in a buffer.
+The i_ prefix means international.
+*********************************************************************/
+
+void i_puts(int msg)
+{
+	eb_puts(i_message(msg));
+}
+
+void i_printf(int msg, ...)
+{
+	const char *realmsg = i_message(msg);
+	va_list p;
+	va_start(p, msg);
+	vprintf(realmsg, p);
+	va_end(p);
+	if (debugFile) {
+		va_start(p, msg);
+		vfprintf(debugFile, realmsg, p);
+		va_end(p);
+	}
+}
+
+// Print and exit.  This puts newline on, like puts.
+void i_printfExit(int msg, ...)
+{
+	const char *realmsg = i_message(msg);
+	va_list p;
+	va_start(p, msg);
+	vprintf(realmsg, p);
+	nl();
+	va_end(p);
+	if (debugFile) {
+		va_start(p, msg);
+		vfprintf(debugFile, realmsg, p);
+		fprintf(debugFile, "\n");
+		va_end(p);
+	}
+	ebClose(99);
+}
+
+/*********************************************************************
+The following error display functions are specific to edbrowse,
+rather than extended versions of the standard unix print functions.
+Thus I don't need the i_ prefix.
+*********************************************************************/
+
+char errorMsg[1024];
+
+// Show the error message, not just the question mark, after these commands.
+static const char showerror_cmd[] = "AbefMqrw^&";
+
+// Set the error message.  Type h to see the message.
+void setError(int msg, ...)
+{
+	va_list p;
+	char *a;		// result of vasprintf
+	int l;
+
+	if (msg < 0) {
+		errorMsg[0] = 0;
+		return;
+	}
+
+	va_start(p, msg);
+	if (vasprintf(&a, i_message(msg), p) < 0)
+		i_printfExit(MSG_MemAllocError, 4096);
+	va_end(p);
+// If the error message is crazy long, truncate it.
+	l = sizeof(errorMsg) - 1;
+	strncpy(errorMsg, a, l);
+	nzFree(a);
+}
+
+void showError(void)
+{
+	if (errorMsg[0])
+		eb_puts(errorMsg);
+	else
+		i_puts(MSG_NoErrors);
+}
+
+void showErrorConditional(char cmd)
+{
+	if (helpMessagesOn || strchr(showerror_cmd, cmd))
+		showError();
+	else
+		eb_puts("?");
+}
+
+void showErrorAbort(void)
+{
+	showError();
+	ebClose(99);
+}
+
+void eb_puts(const char *s)
+{
+	puts(s);
+	if (debugFile)
+		fprintf(debugFile, "%s\n", s);
+}
+
+// the help command
+bool helpUtility(void)
+{
+	int cx;
+
+	if (!cxQuit(context, 0))
+		return false;
+
+	undoSpecialClear();
+
+// maybe we already have a buffer with the help guide in it
+	for (cx = 1; cx < MAXSESSION; ++cx) {
+		Window *w = sessionList[cx].lw;
+		if (!w)
+			continue;
+		if (!w->f0.fileName)
+			continue;
+		if (!stringEqual(w->f0.fileName, "qrg.browse"))
+			continue;
+		cxSwitch(cx, false);
+		i_printf(MSG_MovedSession, cx);
+		return true;
+	}
+
+	cx = sideBuffer(0, qrg_string, -1, "qrg");
+	if (cx == 0)
+		return false;
+	cxSwitch(cx, false);
+	i_printf(MSG_MovedSession, cx);
+	browseCurrentBuffer();
+	cw->dot = 1;
+	return true;
+}
+
