@@ -137,32 +137,30 @@ static char *find_http_header(struct i_get *g, const char *name)
 	if (!h)
 		return NULL;
 	for (s = h; *s; s = v) {
-/* find start of next line */
+// find start of next line
 		v = strchr(s, '\n');
 		if (!v)
 			break;
 		++v;
 
-/* name: value */
+// name: value
 		t = strchr(s, ':');
 		if (!t || t >= v)
 			continue;
 		u = t;
-		while (u > s && isspace(u[-1]))
+		while (u > s && isspaceByte(u[-1]))
 			--u;
 		if (u - s != namelen)
 			continue;
 		if (!memEqualCI(s, name, namelen))
 			continue;
 
-/* This is a match */
+// This is a match
 		++t;
-		while (t < v && isspace(*t))
-			++t;
+		while (t < v && isspaceByte(*t)) ++t;
 		u = v;
-		while (u > t && isspace(u[-1]))
-			--u;
-/* remove quotes */
+		while (u > t && isspaceByte(u[-1])) --u;
+// remove quotes
 		if (u - t >= 2 && *t == u[-1] && (*t == '"' || *t == '\''))
 			++t, --u;
 		if (u == t)
@@ -330,16 +328,19 @@ static CURLcode fetch_internet(struct i_get *g)
 	return curlret;
 }
 
-/* Callback used by libcurl. Captures data from http, ftp, pop3, gopher.
- * download states:
- * -1 user aborted the download
- * 0 standard in-memory download
- * 1 download but stop and ask user if he wants to download to disk
-* 2 disk download in foreground
-* 3 disk download parent thread
-* 4 disk download child thread
-* 5 disk download before the thread is spawned
- * 6 mime type says this should be a stream */
+/*********************************************************************
+Callback used by libcurl. Captures data from http, ftp, pop3, gopher.
+download states, in down_state:
+-1 user aborted the download
+0 standard in-memory download
+1 download but stop and ask user if he wants to download to disk
+2 disk download in foreground
+3 disk download parent thread
+4 disk download child thread
+5 disk download before the thread is spawned
+6 mime type says this should be a stream
+*********************************************************************/
+
 size_t
 eb_curl_callback(char *incoming, size_t size, size_t nitems, struct i_get * g)
 {
@@ -683,8 +684,8 @@ f3:
 	while (*date == ' ')
 		++date;
 	if ((*date == '+' || *date == '-') &&
-	    isdigit(date[1]) && isdigit(date[2]) &&
-	    isdigit(date[3]) && isdigit(date[4])) {
+	    isdigitByte(date[1]) && isdigitByte(date[2]) &&
+	    isdigitByte(date[3]) && isdigitByte(date[4])) {
 		zone = 10 * (date[1] - '0') + date[2] - '0';
 		zone *= 60;
 		zone += 10 * (date[3] - '0') + date[4] - '0';
@@ -727,7 +728,7 @@ bool parseRefresh(char *ref, int *delay_p)
 	if (memEqualCI(u, "url=", 4)) {
 		char qc;
 		u += 4;
-		while (isspace(*u))
+		while (isspaceByte(*u))
 			++u;
 		qc = *u;
 		if (qc == '"' || qc == '\'')
@@ -795,7 +796,7 @@ static void urlSanitize(struct i_get *g, const char *post)
 
 // get rid of : in http://this.that.com:/path, curl can't handle it.
 	getPortLocURL(g->urlcopy, &portloc, 0);
-	if (portloc && !isdigit(portloc[1])) {
+	if (portloc && !isdigitByte(portloc[1])) {
 		const char *s = portloc + strcspn(portloc, "/?#\1");
 		strmove((char *)portloc, s);
 		g->urlcopy_l = strlen(g->urlcopy);
@@ -825,7 +826,7 @@ bool httpConnect(struct i_get *g)
 	uchar sxfirst = 0;
 	int n;
 
-		redirect_count = 0;
+	redirect_count = 0;
 
 	if (!getProtHostURL(url, prot, host)) {
 // only the foreground http thread uses setError,
@@ -1171,10 +1172,10 @@ they go where they go, so this doesn't come up very often.
 		}
 
 		if (g->down_state == 5) {
-/* user has directed a download of this file in the background. */
-/* We spawn a thread to do this, then return, but g could go away */
-/* before the child thread has a chance to read its contents. */
-			struct i_get g0;
+// user has directed a download of this file in the background.
+// We spawn a thread to do this, then return, but g could go away
+// before the child thread has a chance to read its contents.
+			static struct i_get g0;
 			pthread_t tid;
 			nzFree(g->buffer);
 			g->buffer = NULL;
@@ -1450,6 +1451,7 @@ curl_fail:
 	}
 
 	if (g->headers_p) {
+		nzFree(*g->headers_p);
 		*g->headers_p = g->headers;
 // The string is your responsibility now.
 		g->headers = 0;
@@ -1477,7 +1479,7 @@ void *httpConnectBack1(void *ptr)
 	g.tsn = ++tsn;
 	debugPrint(3, "bg thread %d", tsn);
 	i_puts(MSG_DownProgress);
-/* push job onto the list for tracking and display */
+// push job onto the list for tracking and display
 	job = allocMem(sizeof(struct BG_JOB) + strlen(g.down_file));
 	job->state = 4;
 	strcpy(job->file, g.down_file);
@@ -1561,7 +1563,7 @@ void *httpConnectBack3(void *ptr)
 		stringAndString(&a, &l, "\r\n\r\n");
 		stringAndString(&a, &l, outgoing_headers);
 		stringAndString(&a, &l, outgoing_body);
-		while (l && isspace(a[l - 1]))
+		while (l && isspaceByte(a[l - 1]))
 			a[--l] = 0;
 		t->value = a;
 	}
@@ -1687,7 +1689,7 @@ static void ftp_listing(struct i_get *g)
 	stringAndString(&g->buffer, &g->length, "<html>\n<body>\n");
 
 	if (!incomingLen) {
-		i_stringAndMessage(&g->buffer, &g->length, MSG_FTPEmptyDir);
+		stringAndMessage(&g->buffer, &g->length, MSG_FTPEmptyDir);
 	} else {
 
 		s = incomingData;
@@ -1829,7 +1831,7 @@ static void gopher_listing(struct i_get *g)
 	stringAndString(&g->buffer, &g->length, "<html>\n<body>\n");
 
 	if (!incomingLen) {
-		i_stringAndMessage(&g->buffer, &g->length, MSG_GopherEmptyDir);
+		stringAndMessage(&g->buffer, &g->length, MSG_GopherEmptyDir);
 	} else {
 
 		s = incomingData;
@@ -1937,7 +1939,7 @@ void ebcurl_setError(CURLcode curlret, const char *url, int action,
 		break;
 
 	case CURLE_SSL_CONNECT_ERROR:
-		(*fn) (MSG_SSLConnectError, curl_error);
+		(*fn) (MSG_SSLCurlError, curl_error);
 		break;
 
 	case CURLE_LOGIN_DENIED:
@@ -2003,10 +2005,10 @@ static bool ftpConnect(struct i_get *g, char *creds_buf)
 	curlret = fetch_internet(g);
 
 	if (g->down_state == 5) {
-/* user has directed a download of this file in the background. */
-/* We spawn a thread to do this, then return, but g could go away */
-/* before the child thread has a chance to read its contents. */
-		struct i_get g0;
+// user has directed a download of this file in the background.
+// We spawn a thread to do this, then return, but g could go away
+// before the child thread has a chance to read its contents.
+		static struct i_get g0;
 		pthread_t tid;
 		nzFree(g->buffer);
 		g->buffer = NULL;
@@ -2147,7 +2149,7 @@ We don't need to do that either.
 
 	f = fopen(tempfile, (cw->binMode ? "rb" : "r"));
 	if(!f) { // this should never happen
-		setError(MSG_OpenFail, tempfile);
+		setError(MSG_NoOpen, tempfile);
 		goto fail;
 	}
 	curl_easy_setopt(h, CURLOPT_UPLOAD, 1L);
@@ -2207,7 +2209,7 @@ static bool gopherConnect(struct i_get *g)
 		g->down_state = 1;
 // 0 is tricky because "05" and "09" can mean binary
 // in doubt, treat as integer and skip leading 0s
-		while (first == '0' && isdigit(s[2])) {
+		while (first == '0' && isdigitByte(s[2])) {
 			s++;
 			first = s[1];
 		}
@@ -2223,10 +2225,10 @@ static bool gopherConnect(struct i_get *g)
 	curlret = fetch_internet(g);
 
 	if (g->down_state == 5) {
-/* user has directed a download of this file in the background. */
-/* We spawn a thread to do this, then return, but g could go away */
-/* before the child thread has a chance to read its contents. */
-		struct i_get g0;
+// user has directed a download of this file in the background.
+// We spawn a thread to do this, then return, but g could go away
+// before the child thread has a chance to read its contents.
+		static struct i_get g0;
 		pthread_t tid;
 		nzFree(g->buffer);
 		g->buffer = NULL;
@@ -2669,15 +2671,15 @@ static size_t
 curl_header_callback(char *header_line, size_t size, size_t nmemb,
 		     struct i_get *g)
 {
-	const struct MIMETYPE *mt;
+	const struct MIMETYPE *mt = 0;
 	size_t bytes_in_line = size * nmemb;
 	stringAndBytes(&g->headers, &g->headers_len,
 		       header_line, bytes_in_line);
 
 	scan_http_headers(g, true);
-	mt = cf->mt;
 
 // a from-the-web mime type causes a download interrupt
+	if(g->cf) mt = g->cf->mt;
 	if (g->pg_ok && mt && !(mt->down_url | mt->from_file) &&
 	    !(mt->outtype && g->playonly)) {
 		g->down_state = 6;
@@ -2753,7 +2755,7 @@ ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data,
 		char *s;
 // data may not be null terminated; can't use strstr
 		for (s = data; s < data + size - 6; ++s)
-			if (!strncmp(s, " MOVE", 5) && isspace(s[5])) {
+			if (!strncmp(s, " MOVE", 5) && isspaceByte(s[5])) {
 				g->move_capable = true;
 				break;
 			}
@@ -2810,7 +2812,7 @@ top:
 	answer = getFileName(g->down_msg, fp2, false, true);
 /* space for a filename means read into memory */
 	if (stringEqual(answer, " ")) {
-		g->down_state = 0;	/* in memory download */
+		g->down_state = 0;	// in memory download
 		nzFree(fp2);
 		return;
 	}
@@ -2971,6 +2973,7 @@ static bool crossOrigin(Tag *t, const char *url2)
 	const char *url1 = t->f0->hbase;
 	bool u1, u2;
 	const char *host1, *host2;
+	const char *prot2;
 
 // If the bottom url is empty then it is a blank frame, which is fine,
 // or it was some javascript code, which has been moved to another
@@ -3008,6 +3011,10 @@ static bool crossOrigin(Tag *t, const char *url2)
 		return false;
 	}
 
+// the data protocol is always ok
+	prot2 = getProtURL(url2);
+	if(stringEqual(prot2, "data")) return true;
+
 // they are both urls, compare the domains.
 // Neither of them should come out null, but I'll check for that.
 	if(!(host1 = getHostURL(url1))) {
@@ -3212,8 +3219,7 @@ So check for serverData null here. Once again we pop the frame.
 	prepareForBrowse(serverData, serverDataLen);
 	if (javaOK(cf->fileName))
 		createJSContext(cf);
-	nzFree(newlocation);	/* should already be 0 */
-	newlocation = 0;
+	nzFree(newlocation), newlocation = 0;
 
 	start = cw->numTags;
 	cdt = newTag(cf, "Document");
@@ -3288,14 +3294,21 @@ bool reexpandFrame(void)
 	Frame *save_cf = cf;
 	bool rc;
 
-// I don't know why cf would ever not be newloc_f.
 	cf = newloc_f;
 	frametag = cf->frametag;
+// even if this frame was closed, I think we're going to want to see it.
+	frametag->contracted = false;
 	cdt = frametag->firstchild;
 
 // Cut away our tree nodes from the previous document, which are now inaccessible.
 	underKill(cdt);
 
+// Warning: subframes of this frame are not removed.
+// they hang around in the js system until the entire window is removed.
+// Illustrated by jsrt: expand the butterfly frame,  then the rubix cube frame,
+// then replace the butterfly frame and the rubik context doesn't go away.
+// Fix this some day.
+
 	delTimers(cf);
 	freeJSContext(cf);
 	nzFree(cf->dw);
@@ -3310,14 +3323,14 @@ bool reexpandFrame(void)
 	cf->firstURL = 0;
 	rc = readFileArgv(cf->fileName, 2, 0);
 	if (!rc) {
-/* serverData was never set, or was freed do to some other error. */
+// serverData was never set, or was freed do to some other error
 		fileSize = -1;	// don't print 0
 		cf = save_cf;
 		return false;
 	}
 
 	if (serverData == NULL) {
-/* frame replaced itself with a playable stream, what to do? */
+// frame replaced itself with a playable stream, what to do?
 		fileSize = -1;
 		cf = save_cf;
 		return true;
