@@ -359,7 +359,7 @@ static bool parseURL(const char *url, const char **proto, int *prlen, const char
 				break;
 		if (q < p)
 			p = 0;
-		if (isdigit(url[0]))
+		if (isdigitByte(url[0]))
 			p = 0;
 	}
 
@@ -1015,7 +1015,6 @@ squash:
 	goto out_n;
 }
 
-/* This routine could be, should be, more sophisticated */
 bool sameURL(const char *s, const char *t)
 {
 	const char *u, *p, *q;
@@ -1115,6 +1114,7 @@ retry:
 			strmove(buf, s + 1);
 		}
 	}			/* more than ten characters */
+	if(!buf[0]) return 0;
 	return buf;
 }
 
@@ -1549,7 +1549,7 @@ bool receiveCookie(const char *url, const char *str)
 
 	if (!curlActive)
 		return false;
-	debugPrint(3, "cookie %s", str);
+	debugPrint(3, "set cookie %s", str);
 
 	server = getHostURL(url);
 	if (server == 0 || !*server)
@@ -1711,6 +1711,8 @@ void cookiesFromJar(void)
 
 bool isInDomain(const char *d, const char *s)
 {
+// .foo.bar is same as foo.bar
+	if(*d == '.') ++d;
 	int dl = strlen(d);
 	int sl = strlen(s);
 	int j = sl - dl;
@@ -1718,8 +1720,7 @@ bool isInDomain(const char *d, const char *s)
 		return false;
 	if (!memEqualCI(d, s + j, dl))
 		return false;
-	if (j && s[j - 1] != '.')
-		return false;
+	if (j && s[j - 1] != '.') return false;
 	return true;
 }
 
@@ -1743,7 +1744,7 @@ that belong to this web page, including any new cookies that were
 added during this edbrowse session, e.g. document.cookie = newCookie;
 *********************************************************************/
 
-void sendCookies(char **s, int *l, const char *url, bool issecure)
+void findcookies(char **s, int *l, const char *url, bool issecure)
 {
 	const char *server = getHostURL(url);
 	const char *data = getDataURL(url);
@@ -1786,8 +1787,13 @@ void sendCookies(char **s, int *l, const char *url, bool issecure)
  */
 		if (!strncmp(c->domain, httponly_prefix, httponly_prefix_len))
 			continue;
-		if (!isInDomain(c->domain, server))
-			continue;
+		if(c->tail) {
+			if (!isInDomain(c->domain, server))
+				continue;
+		} else {
+			if(!stringEqualCI(c->domain, server))
+				continue;
+		}
 		if (!isPathPrefix(c->path, data))
 			continue;
 		if (c->expires && c->expires < now)
@@ -1802,7 +1808,7 @@ void sendCookies(char **s, int *l, const char *url, bool issecure)
 		stringAndString(s, l, c->name);
 		stringAndChar(s, l, '=');
 		stringAndString(s, l, c->value);
-		debugPrint(3, "send cookie %s=%s", c->name, c->value);
+		debugPrint(3, "find cookie %s=%s", c->name, c->value);
 	}
 
 	if (c != NULL) {
@@ -2981,8 +2987,23 @@ bool runPluginCommand(const struct MIMETYPE * m,
 			return false;
 		}
 		infile = tempin;
-	} else if (inurl)
+	} else if (inurl) {
 		infile = inurl;
+// gather http headers for this url
+		if(isURL(inurl) && curlActive) {
+			bool secure = false;
+			const char *proto;
+			char *cookie;
+			int cook_l;
+			cookie = initString(&cook_l);
+			proto = getProtURL(inurl);
+			if (proto && stringEqualCI(proto, "https"))
+				secure = true;
+			findcookies(&cookie, &cook_l, inurl, secure);
+			setenv("PLUGINHEADERS", cookie, 1);
+			nzFree(cookie);
+		}
+	}
 
 // reserve an output file, whether we need it or not
 	++tempIndex;
@@ -3208,333 +3229,12 @@ native irc. Code taken from https://dl.suckless.org/tools/sic-1.2.tar.gz
 under the MIT license.
 *********************************************************************/
 
-static char irc_in[4096];
-static char irc_out[4096];
-
-static void ircSend(FILE *f, char *fmt, ...)
-{
-	va_list ap;
-	va_start(ap, fmt);
-	vsnprintf(irc_out, sizeof irc_out, fmt, ap);
-	va_end(ap);
-// he doesn't check that fprintf works, that the socket is still there,
-// I guess I won't check either, a broken ssocket will be obvious
-// the next time we try to read.
-// Note that crlf is the standard.
-	fprintf(f, "%s\r\n", irc_out);
-}
-
-// skip ahead to c
-static char*ircSkip(char *s, char c)
-{
-	while(*s != c && *s != '\0')
-		s++;
-	if(*s != '\0')
-		*s++ = '\0';
-	return s;
-}
-
-static char * ircEat(char *s, int (*p)(int), int r)
-{
-// this use to be s, I think it's a bug, I changed it to *s
-	while(*s != '\0' && p(*s) == r)
-		s++;
-	return s;
-}
-
-// This uses addTextToBuffer to place the line, and so,
-// cw has to be set to the receiving window.
-static void ircAddLine(const char *channel, bool show, const char *fmt, ...)
-{
-	va_list ap;
-	va_start(ap, fmt);
-	vsnprintf(irc_out, sizeof irc_out - 1, fmt, ap);
-	va_end(ap);
-	strcat(irc_out, "\n");
-// If leading > I will assume it's a system message,
-// and damn annoying to see the system name before each one of these.
-// If you happen to send out a message that starts with >, well then, false positive.
-	if(channel && show && irc_out[0] != '>') {
-// only if there's room, which there should be!
-		unsigned l1 = strlen(channel), l2 = strlen(irc_out);
-		if(l1 + l2 < sizeof irc_out) {
-// This is quick and dirty, just paste them together.
-// We may do something more nuanced later.
-			strmove(irc_out + l1, irc_out);
-			memcpy(irc_out, channel, l1);
-		}
-	}
-	addTextToBuffer((uchar*)irc_out, strlen(irc_out), cw->dol, false);
-}
-
-static void ircPrepLine(Window *win, Window *wout, char *line)
-{
-	Window *save_cw;
-	FILE *f = win->ircF;
-	char *usr, *par, *txt;
-	usr = win->f0.hbase;
-	if(!line || !*line)
-		return;
-	if(line[0] == ':') {
-		usr = line + 1;
-		line = ircSkip(usr, ' ');
-		if(line[0] == '\0')
-			return;
-		ircSkip(usr, '!');
-	}
-// lines come in from irc ending in crlf
-	ircSkip(line, '\r');
-	par = ircSkip(line, ' ');
-	txt = ircSkip(par, ':');
-	trimWhite(par);
-	if(stringEqual("PONG", line)) {
-		debugPrint(4, "pong back");
-		return;
-	}
-	if(!wout) return; // should never happen
-// going to call ircAddLine below
-	save_cw = cw, cw = wout;
-	if(stringEqual("PRIVMSG", line))
-		ircAddLine(par, win->ircChannels, "<%s> %s", usr, txt);
-	else if(stringEqual("PING", line)) {
-		ircSend(f, "PONG %s", txt);
-		debugPrint(4, "PING PONG %s", txt);
-	}
-	else {
-		ircAddLine(usr, win->ircChannels, ">< %s (%s): %s", line, par, txt);
-		if(stringEqual("NICK", line) && stringEqual(usr, win->ircNick)) {
-			nzFree(win->ircNick);
-			win->ircNick = cloneString(txt);
-		}
-	}
-	cw = save_cw;
-}
-
-// cw is input window
-static void ircMessage(Window *wout, const char *receiver, const char *msg)
-{
-	Window *win = cw;
-	FILE *f = win->ircF;
-	if(!receiver) {
-		debugPrint(1, "No receiver to send to");
-		return;
-	}
-	cw = wout;
-	ircAddLine(receiver, win->ircChannels, "<%s> %s",
-	(win->ircNick ? win->ircNick : emptyString), msg);
-	cw = win;
-	ircSend(f, "PRIVMSG %s :%s", receiver, msg);
-}
-
-// Set the channel on the input side, but it affects the file name on the output side
-// parameter is input window
-static void ircSetChannel(Window *w, const char *channel)
-{
-	Window *w2;
-	char *p;
-	const char *channel0 = (channel ? channel : "irc");
-	nzFree(w->ircChannel), w->ircChannel = cloneString(channel);
-	nzFree(w->f0.fileName);
-	p = allocMem(strlen(channel0) + 6);
-	sprintf(p, "%s send", channel0);
-	w->f0.fileName = p;
-	w2 = sessionList[w->ircOther].lw;
-	if(w2) ircSetFileName(w2);
-}
-
-// parameter is output window
-void ircSetFileName(Window *w)
-{
-	int i, len;
-	char *p;
-	Window *w2;
-	if(!w->ircoMode) return;
-	for(i = 1, len = 0; i < MAXSESSION; ++i) {
-		w2 = sessionList[i].lw;
-		if(!w2 || !w2->irciMode || w2->ircOther != w->sno) continue;
-		if(w2->ircChannel) len += strlen(w2->ircChannel) + 1;
-	}
-	nzFree(w->f0.fileName);
-	if(!len) { w->f0.fileName = cloneString("irc receive"); return; }
-	p = allocMem(len + 8);
-	for(i = 1, *p = 0; i < MAXSESSION; ++i) {
-		w2 = sessionList[i].lw;
-		if(!w2 || !w2->irciMode || w2->ircOther != w->sno) continue;
-		if(w2->ircChannel) {
-			strcat(p, w2->ircChannel);
-			strcat(p, " ");
-		}
-	}
-	strcat(p, "receive");
-	w->f0.fileName = p;
-}
-
-// win is the same as cw
-static void ircPrepSend(Window *win, Window *wout, char *s)
-{
-	char c, *p;
-	FILE *f = win->ircF;
-	if(s[0] == '\0')
-		return;
-	ircSkip(s, '\n');
-	if(s[0] != ':') {
-		ircMessage(wout, win->ircChannel, s);
-		return;
-	}
-	c = *++s;
-	if(c != '\0' && isspace(s[1])) {
-		p = s + 2;
-		switch(c) {
-		case 'j':
-			ircSend(f, "JOIN %s", p);
-// I'm just assuming it works
-				ircSetChannel(win, p);
-			return;
-		case 'l':
-			s = ircEat(p, isspace, 1);
-			p = ircEat(s, isspace, 0);
-			if(!*s)
-				s = (win->ircChannel ? win->ircChannel : emptyString);
-			if(*p)
-				*p++ = '\0';
-			if(!*p)
-				p = "edbrowse irc";
-			ircSend(f, "PART %s :%s", s, p);
-			if(stringEqual(s, win->ircChannel)) {
-// leaving the channel we are currently sending on,
-// what are we suppose to do?
-				ircSetChannel(win, 0);
-			}
-			return;
-		case 'm':
-			s = ircEat(p, isspace, 1);
-			p = ircEat(s, isspace, 0);
-			if(*p)
-				*p++ = '\0';
-			ircMessage(wout, s, p);
-			return;
-		case 's':
-			ircSetChannel(win, p);
-			return;
-		}
-	}
-	ircSend(f, "%s", s);
-}
-
-bool ircWrite(void)
-{
-	Window *nw = sessionList[cw->ircOther].lw;
-	int i;
-	pst s;
-	if(nw && !nw->ircoMode) nw = 0;
-	if(!nw) { // should never happen
-		setError(MSG_TextRec, cw->ircOther);
-		return false;
-	}
-	fileSize = 0;
-	for(i = 1; i <= cw->dol; ++i) {
-		s = fetchLine(i, 1);
-		fileSize += pstLength(s);
-		if(*s != '\n') { // cull empty lines
-//	*** Message to #edbrowse throttled due to flooding
-// Hopefully one second between each line is enough.
-			if(i > 1) sleep(1);
-			ircPrepSend(cw, nw, (char*)s);
-		}
-		free(s);
-	}
-	return true;
-}
-
-// on the input side
-static time_t ircNow;
-static void ircRead0(Window *w)
-{
-	Window *w2;
-	const char *emsg;
-	FILE *f = w->ircF;
-	int fd = fileno(f);
-	int rc;
-
-// find output window; should always be there
-	 w2 = sessionList[w->ircOther].lw;
-	if(w2 && !w2->ircoMode) w2 = 0;
-
-// use select to see if data is available
-	fd_set rd;
-	struct timeval tv;
-top:
-	FD_ZERO(&rd);
-	FD_SET(fd, &rd);
-	tv.tv_sec = 0; // nonblocking
-	tv.tv_usec = 0;
-	rc = select(fd + 1, &rd, 0, 0, &tv);
-	if(rc < 0) {
-// did somebody hit ^c and precisely the wrong time?
-		if(errno == EINTR)
-			return;
-// some other inexplicable error
-		emsg = " irc select error";
-		goto teardown;
-	}
-	if(rc == 0) {
-// sic would ping the host if nothing comes in for 2 minutes; we don't do that.
-// They ping us that's good enough.
-// This is where we would check ircNow against w->ircRespond + 120
-// and ping the host.
-		return;
-	}
-	if(FD_ISSET(fd, &rd)) {
-// this should always happen.
-		w->ircRespond = ircNow;
-		if(fgets(irc_in, sizeof irc_in, f) == NULL) {
-			emsg = " irc connection lost";
-			goto teardown;
-		}
-		ircPrepLine(w, w2, irc_in);
-// is there more data to get?
-		goto top;
-	}
-	return;
-
-teardown:
-	debugPrint(1, "%s%s", (w->ircChannel ? w->ircChannel : "?"), emsg);
-	fclose(f), w->ircF = 0;
-	w->ircOther = 0;
-	w->irciMode = false;
-	nzFree(w->ircNick), w->ircNick = 0;
-	nzFree(w->f0.fileName), w->f0.fileName = 0;
-	nzFree(w->f0.hbase), w->f0.hbase = 0;
-	if(w2) {
-		Window *save_cw = cw;
-		cw = w2;
-		ircAddLine(w->ircChannel, true, emsg);
-		cw = save_cw;
-		if(--w2->ircCount == 0) {
-			w2->ircoMode = false;
-			nzFree(w2->f0.fileName), w2->f0.fileName = 0;
-		}
-	}
-	nzFree(w->ircChannel), w->ircChannel = 0;
-}
-
-void ircRead(void)
-{
-	int i;
-	time(&ircNow);
-	for(i = 1; i < MAXSESSION; ++i) {
-		Window *lw = sessionList[i].lw;
-		if(lw && lw->irciMode)
-			ircRead0(lw);
-	}
-}
-
-// And now for the socket stuff, which I pretty much copied without understanding.
-
 #include <netdb.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
        #include <arpa/inet.h>
+#include <openssl/ssl.h>
+#include <openssl/err.h>	// for error-retrieval
 
 typedef unsigned int IP32bit;
 #define NULL_IP (IP32bit)(-1)
@@ -3557,7 +3257,7 @@ static int tcp_isDots(const char *s)
 				return 0;
 			continue;
 		}
-		if (!isdigit(c))
+		if (!isdigitByte(c))
 			return 0;
 	}
 	return (nd == 3);
@@ -3629,6 +3329,360 @@ static char *tcp_dots_name(const char *s)
 	return tcp_ip_name(tcp_dots_ip(s));
 }
 
+static char irc_in[4096];
+static char irc_out[4096];
+static SSL_CTX *sslcx;		// the overall ssl context for irc sockets
+
+static void ircSend(const Window *win, char *fmt, ...)
+{
+	int l;
+	va_list ap;
+	va_start(ap, fmt);
+	vsnprintf(irc_out, sizeof(irc_out) - 2, fmt, ap);
+	va_end(ap);
+	l = strlen(irc_out);
+	strcpy(irc_out + l, "\r\n");
+	l += 2;
+// He doesn't check for write failure, -1,
+// If the socket drops we'll see it on our next select or read.
+// We also assume it won't do a  partial write, and then expect us to write
+// the rest - i.e. the entire write will work if the socket is valid.
+	if(win->ircSecure)
+		SSL_write(win->irc_ssl, irc_out, l);
+	else
+		write(win->irc_fd, irc_out, l);
+}
+
+// skip ahead to c
+static char*ircSkip(char *s, char c)
+{
+	while(*s != c && *s != '\0')
+		s++;
+	if(*s != '\0')
+		*s++ = '\0';
+	return s;
+}
+
+static char * ircEat(char *s, int (*p)(int), int r)
+{
+// this use to be s, I think it's a bug, I changed it to *s
+	while(*s != '\0' && p(*s) == r)
+		s++;
+	return s;
+}
+
+// This uses addTextToBuffer to place the line, and so,
+// cw has to be set to the receiving window.
+static void ircAddLine(const char *channel, bool show, const char *fmt, ...)
+{
+	va_list ap;
+	va_start(ap, fmt);
+	vsnprintf(irc_out, sizeof irc_out - 1, fmt, ap);
+	va_end(ap);
+	strcat(irc_out, "\n");
+// If leading > I will assume it's a system message,
+// and damn annoying to see the system name before each one of these.
+// If you happen to send out a message that starts with >, well then, false positive.
+	if(channel && show && irc_out[0] != '>') {
+// only if there's room, which there should be!
+		unsigned l1 = strlen(channel), l2 = strlen(irc_out);
+		if(l1 + l2 < sizeof irc_out) {
+// This is quick and dirty, just paste them together.
+// We may do something more nuanced later.
+			strmove(irc_out + l1, irc_out);
+			memcpy(irc_out, channel, l1);
+		}
+	}
+	addTextToBuffer((uchar*)irc_out, strlen(irc_out), cw->dol, false);
+}
+
+static void ircPrepLine(Window *win, Window *wout, char *line)
+{
+	Window *save_cw;
+	char *usr, *par, *txt;
+	usr = win->f0.hbase;
+	if(!line || !*line)
+		return;
+	if(line[0] == ':') {
+		usr = line + 1;
+		line = ircSkip(usr, ' ');
+		if(line[0] == '\0')
+			return;
+		ircSkip(usr, '!');
+	}
+// lines come in from irc ending in crlf
+	ircSkip(line, '\r');
+	par = ircSkip(line, ' ');
+	txt = ircSkip(par, ':');
+	trimWhite(par);
+	if(stringEqual("PONG", line)) {
+		debugPrint(4, "pong back");
+		return;
+	}
+	if(!wout) return; // should never happen
+// going to call ircAddLine below
+	save_cw = cw, cw = wout;
+	if(stringEqual("PRIVMSG", line))
+		ircAddLine(par, win->ircChannels, "<%s> %s", usr, txt);
+	else if(stringEqual("PING", line)) {
+		ircSend(win, "PONG %s", txt);
+		debugPrint(4, "PING PONG %s", txt);
+	}
+	else {
+		ircAddLine(usr, win->ircChannels, ">< %s (%s): %s", line, par, txt);
+		if(stringEqual("NICK", line) && stringEqual(usr, win->ircNick)) {
+			nzFree(win->ircNick);
+			win->ircNick = cloneString(txt);
+		}
+	}
+	cw = save_cw;
+}
+
+// cw is input window
+static void ircMessage(Window *wout, const char *receiver, const char *msg)
+{
+	Window *win = cw;
+	if(!receiver) {
+		debugPrint(1, "No receiver to send to");
+		return;
+	}
+	cw = wout;
+	ircAddLine(receiver, win->ircChannels, "<%s> %s",
+	(win->ircNick ? win->ircNick : emptyString), msg);
+	cw = win;
+	ircSend(win, "PRIVMSG %s :%s", receiver, msg);
+}
+
+// Set the channel on the input side, but it affects the file name on the output side
+// parameter is input window
+static void ircSetChannel(Window *w, const char *channel)
+{
+	Window *w2;
+	char *p;
+	const char *channel0 = (channel ? channel : "irc");
+	nzFree(w->ircChannel), w->ircChannel = cloneString(channel);
+	nzFree(w->f0.fileName);
+	p = allocMem(strlen(channel0) + 6);
+	sprintf(p, "%s send", channel0);
+	w->f0.fileName = p;
+	w2 = sessionList[w->ircOther].lw;
+	if(w2) ircSetFileName(w2);
+}
+
+// parameter is output window
+void ircSetFileName(Window *w)
+{
+	int i, len;
+	char *p;
+	Window *w2;
+	if(!w->ircoMode) return;
+	for(i = 1, len = 0; i < MAXSESSION; ++i) {
+		w2 = sessionList[i].lw;
+		if(!w2 || !w2->irciMode || w2->ircOther != w->sno) continue;
+		if(w2->ircChannel) len += strlen(w2->ircChannel) + 1;
+	}
+	nzFree(w->f0.fileName);
+	if(!len) { w->f0.fileName = cloneString("irc receive"); return; }
+	p = allocMem(len + 8);
+	for(i = 1, *p = 0; i < MAXSESSION; ++i) {
+		w2 = sessionList[i].lw;
+		if(!w2 || !w2->irciMode || w2->ircOther != w->sno) continue;
+		if(w2->ircChannel) {
+			strcat(p, w2->ircChannel);
+			strcat(p, " ");
+		}
+	}
+	strcat(p, "receive");
+	w->f0.fileName = p;
+}
+
+// win is the same as cw
+static void ircPrepSend(Window *win, Window *wout, char *s)
+{
+	char c, *p;
+	if(s[0] == '\0')
+		return;
+	ircSkip(s, '\n');
+	if(s[0] != ':') {
+		ircMessage(wout, win->ircChannel, s);
+		return;
+	}
+	c = *++s;
+	if(c != '\0' && isspaceByte(s[1])) {
+		p = s + 2;
+		switch(c) {
+		case 'j':
+			ircSend(win, "JOIN %s", p);
+// I'm just assuming it works
+				ircSetChannel(win, p);
+			return;
+		case 'l':
+			s = ircEat(p, isspace, 1);
+			p = ircEat(s, isspace, 0);
+			if(!*s)
+				s = (win->ircChannel ? win->ircChannel : emptyString);
+			if(*p)
+				*p++ = '\0';
+			if(!*p)
+				p = "edbrowse irc";
+			ircSend(win, "PART %s :%s", s, p);
+			if(stringEqual(s, win->ircChannel)) {
+// leaving the channel we are currently sending on,
+// what are we suppose to do?
+				ircSetChannel(win, 0);
+			}
+			return;
+		case 'm':
+			s = ircEat(p, isspace, 1);
+			p = ircEat(s, isspace, 0);
+			if(*p)
+				*p++ = '\0';
+			ircMessage(wout, s, p);
+			return;
+		case 's':
+			ircSetChannel(win, p);
+			return;
+		}
+	}
+	ircSend(win, "%s", s);
+}
+
+bool ircWrite(void)
+{
+	Window *nw = sessionList[cw->ircOther].lw;
+	int i;
+	pst s;
+	if(nw && !nw->ircoMode) nw = 0;
+	if(!nw) { // should never happen
+		setError(MSG_TextRec, cw->ircOther);
+		return false;
+	}
+	fileSize = 0;
+	for(i = 1; i <= cw->dol; ++i) {
+		s = fetchLine(i, 1);
+		fileSize += pstLength(s);
+		if(*s != '\n') { // cull empty lines
+//	*** Message to #edbrowse throttled due to flooding
+// Hopefully one second between each line is enough.
+			if(i > 1) sleep(1);
+			ircPrepSend(cw, nw, (char*)s);
+		}
+		free(s);
+	}
+	return true;
+}
+
+// on the input side
+static time_t ircNow;
+static void ircRead0(Window *w)
+{
+	Window *w2;
+	const char *emsg;
+	int fd = w->irc_fd, rc;
+
+// find output window; should always be there
+	 w2 = sessionList[w->ircOther].lw;
+	if(w2 && !w2->ircoMode) w2 = 0;
+
+// use select to see if data is available
+	fd_set rd;
+	struct timeval tv;
+top:
+	FD_ZERO(&rd);
+	FD_SET(fd, &rd);
+	tv.tv_sec = 0; // nonblocking
+	tv.tv_usec = 0;
+	rc = select(fd + 1, &rd, 0, 0, &tv);
+	if(rc < 0) {
+// did somebody hit ^c and precisely the wrong time?
+		if(errno == EINTR)
+			return;
+// some other inexplicable error
+		emsg = " irc select error";
+		goto teardown;
+	}
+	if(rc == 0) {
+// ping the host if nothing has come in for 10 minutes
+		if(ircNow - w->ircRespond >= 500 && !w->ircPingOut) {
+			debugPrint(4, "pingout %s", w->f0.hbase);
+			ircSend(w, "PING %s", w->f0.hbase);
+			w->ircPingOut = true;
+		} else if(ircNow - w->ircRespond >= 550 && w->ircPingOut) {
+			emsg = " irc server timeout";
+			goto teardown;
+		}
+		return;
+	}
+	if(FD_ISSET(fd, &rd)) {
+// this should always happen.
+		unsigned pos = 0;
+		int n;
+		char *linebreak;
+		w->ircRespond = ircNow;
+		w->ircPingOut = 0;
+nextread:
+		if(w->ircSecure)
+			n = SSL_read(w->irc_ssl, irc_in + pos, sizeof(irc_in) - pos - 1);
+		else
+			n = read(fd, irc_in + pos, sizeof(irc_in) - pos - 1);
+		if(n <= 0) {
+			emsg = " irc connection lost";
+			goto teardown;
+		}
+// null terminate so it is a string
+		irc_in[pos += n] = 0;
+nextline:
+		linebreak = strchr(irc_in, '\n');
+		if(!linebreak) { // still don't have a complete line
+			if(pos < sizeof(irc_in) - 1) goto nextread; // get the rest of the line
+// Oops irc_in is full and still no nl. Just force it.
+// This should never happen, servers have a limit on line length.
+			irc_in[pos - 2] = '\r';
+			irc_in[pos - 1] = '\n';
+			linebreak = irc_in + pos - 1;
+		}
+		ircPrepLine(w, w2, irc_in);
+		++linebreak;
+		if(!*linebreak) goto top; // select again
+// There is another line or another partial line.
+		strmove(irc_in, linebreak);
+		pos = strlen(irc_in);
+		goto nextline;
+	}
+	return;
+
+teardown:
+	debugPrint(1, "%s%s", (w->ircChannel ? w->ircChannel : "?"), emsg);
+	if(w2) {
+		Window *save_cw = cw;
+		cw = w2;
+		ircAddLine(w->ircChannel, true, emsg);
+		cw = save_cw;
+		if(--w2->ircCount == 0) {
+			w2->ircoMode = false;
+			nzFree(w2->f0.fileName), w2->f0.fileName = 0;
+			freeWindowLines(w2->r_map);
+			w2->r_map = 0;
+		} else {
+// I have to clear the channel here so the file name comes out right.
+			nzFree(w->ircChannel), w->ircChannel = 0;
+			ircSetFileName(w2);
+		}
+	}
+	ircClose(w);
+}
+
+void ircRead(void)
+{
+	int i;
+	time(&ircNow);
+	for(i = 1; i < MAXSESSION; ++i) {
+		Window *lw = sessionList[i].lw;
+		if(lw && lw->irciMode)
+			ircRead0(lw);
+	}
+}
+
 static int ircDial(char *host, int port)
 {
 	static struct addrinfo hints;
@@ -3715,11 +3769,11 @@ bool ircSetup(char *line)
 	line += 3;
 	if(!*line) goto usage;
 	spaceCrunch(line, true, false);
-	if(!isdigit(*line)) goto usage;
+	if(!isdigitByte(*line)) goto usage;
 	cxin = strtol(line, &line, 10);
 	if(cxin < 0 || *line != ' ') goto usage;
 	++line;
-	if(!isdigit(*line)) goto usage;
+	if(!isdigitByte(*line)) goto usage;
 	cxout = strtol(line, &line, 10);
 	if(cxout < 0 || *line != ' ') goto usage;
 	++line;
@@ -3755,7 +3809,7 @@ bool ircSetup(char *line)
 		setError(MSG_IrcCompat, cxout);
 		return false;
 	}
-	if(!isalpha(*line)) goto usage;
+	if(!isalphaByte(*line)) goto usage;
 	domain = line;
 	nick = ircSkip(line, ' ');
 	if(!*nick) goto usage;
@@ -3764,11 +3818,17 @@ bool ircSetup(char *line)
 	password = ircSkip(nick, ':');
 	if(!*password) password = 0;
 	p = ircSkip(domain, ':');
+	win->ircSecure = false;
 	if(*p) {
-		port = stringIsNum(p);
-		if(port < 0) {
-			setError(MSG_BadPort);
-			return false;
+		if(*p == '*' && p[1] == 0) {
+			port = 6697, win->ircSecure = true;
+		} else {
+			if(*p == '*') ++p, win->ircSecure = true;
+			port = strtol(p, &p, 10);
+			if(port <= 0 || port > 65535 || *p) {
+				setError(MSG_BadPort);
+				return false;
+			}
 		}
 	}
 
@@ -3779,7 +3839,50 @@ bool ircSetup(char *line)
 
 	fd = ircDial(domain, port);
 	if(fd < 0) return false;
-	win->ircF = f = fdopen(fd, "r+");
+	if(win->ircSecure) {
+		static bool first = true;
+		if(first) {
+			first = false;
+/*********************************************************************
+I am concerned about initializing and using openssl, for these raw irc sockets,
+while curl initializes and uses openssl for its secure protocols.
+Will there be collisions? Inconsistencies?
+Will this induce bugs that are very difficult to reproduce?
+*********************************************************************/
+			SSLeay_add_ssl_algorithms();
+			sslcx = SSL_CTX_new(SSLv23_client_method());
+			SSL_CTX_set_options(sslcx, SSL_OP_ALL);
+			SSL_CTX_load_verify_locations(sslcx, sslCerts, NULL);
+			SSL_CTX_set_default_verify_paths(sslcx);
+			SSL_CTX_set_mode(sslcx, SSL_MODE_AUTO_RETRY);
+		}
+// should we verify the certificate?
+		if(mustVerifyHost(domain))
+			SSL_CTX_set_verify(sslcx, SSL_VERIFY_PEER, NULL);
+		else
+			SSL_CTX_set_verify(sslcx, SSL_VERIFY_NONE, NULL);
+// create the secure stream
+		SSL* secstream = SSL_new(sslcx);
+		SSL_set_fd(secstream, fd);
+/* Do we need this?
+		secstream->options |= SSL_OP_NO_TLSv1;
+*/
+		int err = SSL_connect(secstream);
+		if (err != 1) {
+			err = ERR_peek_last_error();
+			ERR_clear_error();
+			SSL_free(secstream);
+			close(fd);
+			if (ERR_GET_REASON(err) == SSL_R_CERTIFICATE_VERIFY_FAILED)
+				setError(MSG_NoCertify, domain);
+			else
+				setError(MSG_SSLConnectError, err);
+			return false;
+		}
+		win->irc_ssl = secstream;
+	}
+
+	win->irc_fd = fd;
 	win->irciMode = true;
 	wout->ircoMode = true;
 	wout->ircCount++;
@@ -3788,23 +3891,23 @@ bool ircSetup(char *line)
 	nzFree(win->f0.hbase);
 	win->f0.hbase = cloneString(domain);
 	ircSetChannel(win, 0);
+	win->ircRespond = ircNow;
+	win->ircPingOut = false;
 
 	// login
 	if(password) {
-		ircSend(f, "PASS %s", password);
+		ircSend(win, "PASS %s", password);
 		debugPrint(4, "PASS %s", password);
 	}
-	ircSend(f, "NICK %s", nick);
+	ircSend(win, "NICK %s", nick);
 	debugPrint(4, "NICK %s", nick);
-	ircSend(f, "USER %s localhost %s :%s", nick, domain, nick);
+	ircSend(win, "USER %s localhost %s :%s", nick, domain, nick);
 	debugPrint(4, "USER %s localhost %s :%s", nick, domain, nick);
 	if(join) {
-		ircSend(f, "JOIN %s", join);
+		ircSend(win, "JOIN %s", join);
 		debugPrint(4, "JOIN %s", join);
 		ircSetChannel(win, join);
 	}
-	fflush(f);
-	setbuf(f, NULL);
 // And hopefully that worked.
 	return true;
 
@@ -3813,6 +3916,20 @@ usage:
 	return false;
 }
 
+void ircClose(Window *w)
+{
+	w->irciMode = w->ircChannels = w->ircSecure = false;
+	w->ircOther = 0;
+	if(w->irc_fd) close(w->irc_fd);
+	w->irc_fd = 0;
+	if(w->irc_ssl) SSL_free(w->irc_ssl);
+	w->irc_ssl = 0;
+	nzFree(w->ircNick), w->ircNick = 0;
+	nzFree(w->ircChannel), w->ircChannel = 0;
+	nzFree(w->f0.fileName), w->f0.fileName = 0;
+	nzFree(w->f0.hbase), w->f0.hbase = 0;
+}
+
 /*********************************************************************
 irc is based on polling, just another interval timer, like javascript timers.
 This allows us to reuse that machinery.
@@ -3836,7 +3953,7 @@ If you change anything in a buffer, and irc adds something to its output
 buffer at the same time, bad things happen.  BOOM!
 It would take a lot of work to go down this path, but the one I'm on
 is also a bit dangerous.
-Use the unix timer to wake up every 3 seconds and poll irc,
+Use the unix timer to wake up every few seconds and poll irc,
 but only while readline() has control.
 This should get around the threadsafe issues.
 Only one "thread" is managing buffers at a time. We hope.
@@ -3850,7 +3967,7 @@ static void ircAlarm(int n)
 	ircAlarming = true;
 	ircRead();
 	signal(SIGALRM, ircAlarm);
-	alarm(3);
+	alarm(7);
 	ircAlarming = false;
 }
 
