@@ -13,11 +13,11 @@ Frame *cf;
 int gfsn; // global frame sequence number
 const char *progname;
 const char eol[] = "\r\n";
-const char *version = "3.8.5";
+const char *version = "3.8.6";
 char *changeFileName;
 char *configFile, *addressFile, *cookieFile, *emojiFile;
 char *mailDir, *mailUnread, *mailStash, *mailReply;
-char *recycleBin, *sigFile, *sigFileEnd;
+char *sigFile, *sigFileEnd;
 char *cacheDir;
 int cacheSize = 1000, cacheCount = 10000;
 char *ebTempDir, *ebUserDir;
@@ -28,12 +28,14 @@ char selsep = ',';
 bool allowRedirection = true, allowJS = true, sendReferrer = true;
 bool blockJS;
 bool ftpActive;
+bool errorExit;
 int webTimeout = 20, mailTimeout = 0;
 int displayLength = 500;
 int verifyCertificates = 1;
 char *sslCerts, *pubKey;
 int localAccount, maxAccount;
 struct MACCOUNT accounts[MAXACCOUNT];
+bool ebvar = true; // update environment variables before each shell command
 bool flow = true; // send mail with format=flowed
 int maxMime;
 struct MIMETYPE mimetypes[MAXMIME];
@@ -44,7 +46,6 @@ time_t intStart;
 bool ismc, isimap, passMail;
 // next two variables work around curl bug 7284
 bool inInput, inInitFunction, listNA;
-long long fileSize;
 char *dbarea, *dblogin, *dbpw;	// to log into the database
 bool fetchBlobColumns;
 bool caseInsensitive, searchStringsAll, searchWrap = true, ebre = true;
@@ -52,6 +53,7 @@ bool binaryDetect = true;
 bool inputReadLine;
 bool curlAuthNegotiate = false;
 int context = 1;
+int cx_previous;
 pst linePending;
 struct ebSession sessionList[MAXSESSION], *cs;
 int maxSession;
@@ -118,17 +120,19 @@ static void finishBrowse(void)
 	cw->browseMode = cf->browseMode = true;
 }
 
-/* Catch interrupt and react appropriately. */
+// Catch interrupt and react appropriately
 static void catchSig(int n)
 {
 	time_t now;
 	int d;
 	pthread_t t1 = foreground_thread, t2;
 	intFlag = true;
-/* If we were reading from a file, or socket, this signal should
- * cause the read to fail.  Check for intFlag, so we know it was
- * interrupted, and not an io failure.
- * Then clean up appropriately. */
+// If we were reading from a file, or socket, this signal should
+//cause the read to fail.  Check for intFlag in that routine,
+// so we know it was interrupted, and not an io failure.
+// Then clean up appropriately.
+// That doesn't concern us here, except that the read should fail
+// and control should return quickly to the user.
 	signal(SIGINT, catchSig);
 	if (inInput) {
 		i_puts(MSG_EnterInterrupt);
@@ -154,7 +158,11 @@ static void catchSig(int n)
 			return;	// didn't work
 		pthread_exit(NULL);
 	} else {
+// When would this ever happen? Maybe downloading another js file in background.
 		pthread_kill(t1, SIGINT);
+// We recaught SIGINT on the current thread, so this really does kill the
+// forground thread. But then what happens when this download finishes?
+// Nobody is running inputForever(). I don't know.
 	}
 }
 
@@ -189,6 +197,13 @@ void ebClose(int n)
 	exit(n);
 }
 
+// error exit check function
+void eeCheck(void)
+{
+	if (errorExit)
+		ebClose(1);
+}
+
 static struct ebhost {
 // j = nojs, v = novs, p = proxy, f = function,
 // s = subject, t = to, r = reply, a = agentsite,
@@ -948,11 +963,11 @@ int runEbFunction(const char *line)
 
 // local copies of settings, to restore after function runs.
 	struct {
-		bool rl, endm, lna, H, ci, sg, su8, sw, ebre, bd, iu, hf, hr, vs, sr, can, ftpa, bg, jsbg, js, showall, pg, fbc, ls_reverse, fllo, dno, flow;
+		bool rl, endm, lna, H, ci, sg, su8, sw, ebre, bd, iu, hf, hr, vs, sr, can, ftpa, bg, jsbg, js, showall, pg, fbc, ls_reverse, fllo, dno, ebvar, flow;
 		uchar dw, ls_sort;
 		char lsformat[12], showProgress;
 		char *currentAgent;
-		int agentIndex, debugLevel, timerspeed;
+		int agentIndex, context, cx_previous, debugLevel, timerspeed;
 		int ll, fll;
 	} save;
 
@@ -992,10 +1007,13 @@ int runEbFunction(const char *line)
 	if(!stringEqual(linecopy, "init") && strncmp(linecopy, "set", 3)) {
 		restore = true;
 		save.currentAgent = currentAgent, save.agentIndex = agentIndex;
+		save.context = context;
+		save.cx_previous = cx_previous, cx_previous = 0;
 		save.debugLevel = debugLevel;
 		save.timerspeed = timerspeed;
 		save.dw = dirWrite;
 		save.dno = dno;
+		save.ebvar = ebvar;
 		save.flow = flow;
 		save.ls_sort = ls_sort;
 		strcpy(save.lsformat, lsformat);
@@ -1160,10 +1178,12 @@ done:
 	nzFree(allargs);
 	if(restore) {
 		currentAgent = save.currentAgent, agentIndex = save.agentIndex;
+		cx_previous = (context == save.context) ? save.cx_previous : save.context;
 		debugLevel = save.debugLevel;
 		timerspeed = save.timerspeed;
 		dirWrite = save.dw;
 		dno = save.dno;
+		ebvar = save.ebvar;
 		flow = save.flow;
 		ls_sort = save.ls_sort;
 		strcpy(lsformat, save.lsformat);
