@@ -35,6 +35,9 @@ extern const char *msg_ru[], ebrc_ru[], qrg_ru[];
 // Italian, Enrico Mioso, mrkiko.rs@gmail.com
 extern const char *msg_it[], ebrc_it[], qrg_it[];
 
+// Spanish, Oriol Prieto, kimicaprieto@hotmail.com
+extern const char *msg_es[], ebrc_es[], qrg_es[];
+
 /*********************************************************************
 This file and lang/msg-en line up.
 The constant at line 101 of messages.h corresponds
@@ -92,9 +95,6 @@ this comment block is just the right size.
 *
 *
 *
-*
-*
-*
 *********************************************************************/
 
 enum {
@@ -246,7 +246,7 @@ enum {
 	MSG_NoParen,
 	MSG_EmptyBuffer,
 	MSG_RexpError,
-	MSG_RexpError2,
+	MSG_GetLocalFrag,
 	MSG_NotFound,
 	MSG_LineHigh,
 	MSG_LineLow,
@@ -332,7 +332,7 @@ enum {
 	MSG_Unbrowsable,
 	MSG_BrowseAlready,
 	MSG_NoLabel2,
-	MSG_BrowseI,
+	MSG_ABMalformed,
 	MSG_InsertFunction,
 	MSG_LineUpdateRange,
 	MSG_CNYI,
@@ -376,12 +376,12 @@ enum {
 	MSG_NoRename,
 	MSG_InputNull2,
 	MSG_InputNewline2,
-	MSG_notused279,
-	MSG_notused280,
-	MSG_notused281,
-	MSG_notused282,
-	MSG_notused283,
-	MSG_notused284,
+	MSG_SSLConnectError,
+	MSG_NoUp,
+	MSG_NoDown,
+	MSG_IrcCommandS,
+	MSG_BadUtf8,
+	MSG_GetFrag,
 	MSG_WebRead,
 	MSG_BadURL,
 	MSG_DownSuccess,
@@ -421,7 +421,7 @@ enum {
 	MSG_ABMailLong,
 	MSG_ABNoAt,
 	MSG_ABMailSpaces,
-	MSG_AbMailUnprintable,
+	MSG_ABMailUnprintable,
 	MSG_ABUnterminated,
 	MSG_NoFolderMatch,
 	MSG_ManyFolderMatch,
@@ -695,7 +695,7 @@ enum {
 	MSG_NullCharInList,
 	MSG_NullPtr,
 	MSG_IoctlError,
-	MSG_OpenFail,
+	MSG_notused598,
 	MSG_CreateFail,
 	MSG_InvalidFopen,
 	MSG_BadDirSlash,
@@ -750,7 +750,7 @@ enum {
 	MSG_Fkeys,
 	MSG_FetchBlobOff,
 	MSG_FetchBlobOn,
-	MSG_SSLConnectError,
+	MSG_SSLCurlError,
 	MSG_InputTTY,
 	MSG_InputReadLine,
 	MSG_DebugTraceOff,
@@ -813,4 +813,7 @@ enum {
 	MSG_ManyButtons,
 	MSG_ManyImages,
 	MSG_hold715,
+	MSG_GetFrag2,
+	MSG_UpdateEBVarOff,
+	MSG_UpdateEBVarOn,
 };
