@@ -20,7 +20,15 @@ bool demin = false;
 bool gotimers = true;
 bool uvw;
 int timerspeed = 1;
+long long fileSize;
 char *downDir, *home;
+bool endMarks;		// ^ $ on listed lines
+uchar dirWrite;		// directories read write
+bool dno; // directory names only
+char *recycleBin;
+uchar ls_sort;		// sort method for directory listing
+bool ls_reverse;		// reverse sort
+char lsformat[12];	// size date etc on a directory listing
 
 /*********************************************************************
 Allocate and manage memory.
@@ -258,19 +266,6 @@ void stringAndNum(char **s, int *l, int n)
 	stringAndString(s, l, a);
 }
 
-/* 64M 16K etc */
-void stringAndKnum(char **s, int *l, int n)
-{
-	char a[16];
-	if (n && n / (1024 * 1024) * (1024 * 1024) == n)
-		sprintf(a, "%dM", n / (1024 * 1024));
-	else if (n && n / 1024 * 1024 == n)
-		sprintf(a, "%dK", n / 1024);
-	else
-		sprintf(a, "%d", n);
-	stringAndString(s, l, a);
-}
-
 char *cloneString(const char *s)
 {
 	char *t;
@@ -300,7 +295,7 @@ void leftClipString(char *s)
 	if (!s)
 		return;
 	for (t = s; *t; ++t)
-		if (!isspace(*t))
+		if (!isspaceByte(*t))
 			break;
 	if (t > s)
 		strmove(s, t);
@@ -360,28 +355,28 @@ int stringIsNum(const char *s)
 
 bool stringIsDate(const char *s)
 {
-	if (!isdigit(*s))
+	if (!isdigitByte(*s))
 		return false;
 	++s;
-	if (isdigit(*s))
+	if (isdigitByte(*s))
 		++s;
 	if (*s++ != '/')
 		return false;
-	if (!isdigit(*s))
+	if (!isdigitByte(*s))
 		return false;
 	++s;
-	if (isdigit(*s))
+	if (isdigitByte(*s))
 		++s;
 	if (*s++ != '/')
 		return false;
-	if (!isdigit(*s))
+	if (!isdigitByte(*s))
 		return false;
 	++s;
-	if (isdigit(*s))
+	if (isdigitByte(*s))
 		++s;
-	if (isdigit(*s))
+	if (isdigitByte(*s))
 		++s;
-	if (isdigit(*s))
+	if (isdigitByte(*s))
 		++s;
 	if (*s)
 		return false;
@@ -973,7 +968,7 @@ void camelCase(char *s)
 {
 	char *t, *w;
 	for (t = w = s; *t; ++t)
-		if (*t == '-' && isalpha(t[1]))
+		if (*t == '-' && isalphaByte(t[1]))
 			t[1] = toupper(t[1]);
 		else
 			*w++ = *t;
@@ -1127,7 +1122,7 @@ char *conciseSize(size_t n)
 
 char *conciseTime(time_t t)
 {
-	static char buffer[20];
+	static char buffer[22];
 	static const char *const englishMonths[] = {
 		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
 		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
@@ -1148,7 +1143,10 @@ char *conciseTime(time_t t)
 	static const char *const italMonths[] = {
 		"Gen", "Feb", "Mar", "Apr", "Mag", "Giu",
 		"Lug", "Ago", "Set", "Ott", "Nov", "Dic",
-		0
+	};
+	static const char *const spanishMonths[] = {
+		"Ene", "Feb", "Mar", "Abr", "May",
+		"Jun", "Jul", "Ago", "Sep", "Oct", "Nov", "Dic"
 	};
 	static const char *const *const allMonths[] = { 0,
 		englishMonths,
@@ -1158,6 +1156,7 @@ char *conciseTime(time_t t)
 		germanMonths,
 		englishMonths,
 		italMonths,
+		spanishMonths,
 	};
 	struct tm *tm = localtime(&t);
 	sprintf(buffer, "%s %2d %d %02d:%02d",
@@ -1493,7 +1492,7 @@ char *dirSuffix2(int n, const char *path)
 // names only, don't have file type information, have to go get it
 	t = suffix;
 	ftype = fileTypeByName(path, 1);
-	if(isupper(ftype)) *t++ = '@', ftype = tolower(ftype);
+	if(isupperByte(ftype)) *t++ = '@', ftype = tolower(ftype);
 	c = 0;
 	if (ftype == 'd') c = '/';
 	if (ftype == 's') c = '^';
@@ -1618,9 +1617,6 @@ struct DSR {
 	} u;
 };
 static struct DSR *dsr_list;
-uchar ls_sort;		// sort method for directory listing
-bool ls_reverse;		// reverse sort
-char lsformat[12];	// size date etc on a directory listing
 
 // compare routine for quicksort directory scan, size or time
 static int dircmp_st(const void *s, const void *t)
@@ -1965,10 +1961,9 @@ unlink:
 }
 
 // Move or copy files from one directory to another
-bool moveFiles(int start, int end, int dest, char origcmd)
+bool moveFiles(int start, int end, int dest, char origcmd, char relative)
 {
-	Window *cw1 = cw;
-	Window *cw2 = sessionList[dest].lw;
+	Window *cw1 = cw, *cw2, *w;
 	char *path1, *path2;
 	int ln, cnt, dol;
 
@@ -1977,6 +1972,53 @@ bool moveFiles(int start, int end, int dest, char origcmd)
 		return false;
 	}
 
+	if(!relative) {
+				if (dest >= MAXSESSION) {
+					setError(MSG_SessionHigh, dest, MAXSESSION - 1);
+					return false;
+				}
+				if (dest == 0) {
+					setError(MSG_Session0);
+					return false;
+				}
+				if (dest == context) {
+					setError(MSG_SessionCurrent, dest);
+					return false;
+				}
+				if (!(cw2 = sessionList[dest].lw) || !cw2->dirMode) {
+					char numstring[12];
+					sprintf(numstring, "%d", dest);
+					setError(MSG_NotDir, numstring);
+					return false;
+				}
+	}
+
+	if(relative && !dest) {
+		setError(MSG_NoChange);
+		return false;
+	}
+
+	if(relative == '+') {
+		for(cnt = 0, w = cw; w; w = w->prev, ++cnt) ;
+		if(dest >= cnt) {
+			setError(MSG_NoUp);
+			return false;
+		}
+		for(w = cw; dest; w = w->prev, --dest)  ;
+		cw2 = w;
+	}
+
+	if(relative == '-') {
+		for(cnt = 0, w = cs->lw2; w; w = w->prev, ++cnt) ;
+		if(dest > cnt) {
+			setError(MSG_NoDown);
+			return false;
+		}
+		dest -= cnt;
+		for(w = cs->lw2; dest; w = w->prev, --dest)  ;
+		cw2 = w;
+	}
+
 	ln = start;
 	cnt = end - start + 1;
 	while (cnt--) {
@@ -1997,7 +2039,6 @@ bool moveFiles(int start, int end, int dest, char origcmd)
 		debugPrint(1, "%s%s %s %s",
 		file, ftype, (origcmd == 'm' ? "→" : "≡"), cw2->baseDirName);
 
-
 		cw = cw2;
 		path2 = makeAbsPath(file);
 		cw = cw1;
@@ -2088,7 +2129,10 @@ moved:
 		addTextToBuffer((pst)file, t-file, dol, false);
 		free(file);
 		cw->dot = ++dol;
-		cw->dmap = reallocMem(cw->dmap, DTSIZE * (dol + 1));
+		if(cw->dmap)
+			cw->dmap = reallocMem(cw->dmap, DTSIZE * (dol + 1));
+		else
+			cw->dmap = allocZeroMem(DTSIZE * (dol + 1));
 		memset(cw->dmap + DTSIZE*dol, 0, DTSIZE);
 		cw->dmap[DTSIZE*dol] = ftype[0];
 		if(ftype[0])
@@ -2100,7 +2144,8 @@ moved:
 			memset(cw->r_map + dol, 0, LMSIZE*2);
 			cw->r_map[dol].text = (uchar*)emptyString;
 		}
-		cw = cw1; // put it back
+		cw = cw1; // go back to original window
+		if(origcmd == 't') cw->dot = ln++;
 	}
 
 	return true;
@@ -2138,9 +2183,9 @@ static bool envExpand(const char *line, const char **expanded)
 		goto dollars;
 
 	l = 0;
-	for (s = line + 1; isalnum(*s) || *s == '_'; ++s)
+	for (s = line + 1; isalnumByte(*s) || *s == '_'; ++s)
 		++l;
-	if ((unsigned)l >= sizeof(var1) || isdigit(line[1]) || (*s && *s != '/')) {
+	if ((unsigned)l >= sizeof(var1) || isdigitByte(line[1]) || (*s && *s != '/')) {
 /* invalid syntax, put things back */
 		s = line;
 		goto dollars;
@@ -2338,7 +2383,7 @@ FILE *efopen(const char *name, const char *mode)
 		return f;
 
 	if (*mode == 'r')
-		i_printfExit(MSG_OpenFail, name);
+		i_printfExit(MSG_NoOpen, name);
 	else if (*mode == 'w' || *mode == 'a')
 		i_printfExit(MSG_CreateFail, name);
 	else
@@ -2355,7 +2400,7 @@ int eopen(const char *name, int mode, int perms)
 	if (mode & O_WRONLY)
 		i_printfExit(MSG_CreateFail, name);
 	else
-		i_printfExit(MSG_OpenFail, name);
+		i_printfExit(MSG_NoOpen, name);
 	return -1;
 }
 
